! ============================================================	
! ============================================================	
! polytope.F90      minimizes a function using the polytope method
! ==========================================================
! simplex algorithm to find optimal x
! ==========================================================

subroutine polytope01(xinitial, v, fv)
    use parameters_polytope1
	use numerical_libraries	
	implicit none
	
	! external variables
	real(8), dimension(n1), intent(in) :: xinitial
	real(8), dimension(n1, np1), intent(out) :: v
	real(8), dimension(np1), intent(out) :: fv

	! internal variables
	real(8), dimension(n1) :: x, xin
	integer(4) :: func_evals, itercount
	real(8) :: fvguess				!accumulates results from each polytope
	real(8), dimension(n1) :: vguess
	integer(4), dimension(np1) :: permute	!permutation matrix
	character (len = 30):: how, success
	character (len = 40):: out_success
	integer(4):: j		!counts how many times polytope has been run
    logical :: cosucc

	! ============================================================	
	! SET INITIAL VALUES
	! ==========================================================
	x = xinitial
	xin = xinitial
	v = 0.d0
	fv = 0.d0

	! =================================================================
	! RUN FIRST POLYTOPE
	! =================================================================
    ! calculate function and store results with initial guess
	v(:, 1) = xin
	call spread1(x, fv(1), cosucc)
    
	! construct simplex
	call ctr_simplex01(xin, v, fv)
	
	! sort results
	permute = (/ (j, j=1,np1) /)
	call DSVRGP(np1, fv, fv, permute)		!sorts fv returning both sorted vector fv and permutation vector
	call DPERMA(n1, np1, v, n1, permute, two, v, n1)	!permutates the columns of v according to the sorting of fv

	! =======================================================================================
	! run polyloop
	how = "initial"
	itercount = 1
	func_evals = np1
	
	call poly_loop01(func_evals, itercount, v, fv, how, success)
	
end subroutine polytope01

! ============================================================	
! ============================================================	
! END POLYTOPE EXECUTION
! (secondary subroutines below)
! ==========================================================
! ==========================================================

	subroutine ctr_simplex01(xin, v, fv)
		use parameters_polytope1
		use parameters_quality
        use parameters_firm
        implicit none
		
		! external variables
		real(8), dimension(n1), intent(in) :: xin
		real(8), dimension(n1, n1+1), intent(inout) :: v
		real(8), dimension(n1+1), intent(inout) :: fv

		! internal variables
		real(8), dimension(n1) :: y
		integer(4) :: j
        logical :: cosucc
		
        integer(4), dimension(sizef) :: oldposq
	    integer(4), dimension(sizef) :: olddiscd	
        real(8), dimension(sizeq) :: oldpq
	    real(8), dimension(sizeq) :: oldchiq	
		
        ! ===========================================================
        ! use original choices and aggregates to estimate each deviation
        oldposq = posqomega; olddiscd = discreted; oldpq = priceqh; oldchiq = intermediatechi  
		! ------------------------------------------------------
        do j = 1, n1
			posqomega = oldposq; discreted = olddiscd; priceqh = oldpq; intermediatechi = oldchiq 
            
            ! new guess of parameters
            y = xin
			if (y(j) /= 0) then
                y(j) = y(j) + usual_delta(j)
            else
				y(j) = zero_term_delta 
            end if
            
            ! calculate objective and repeat with smalle change in parameter if unsuccessful
			call spread1(y, fv(j+1), cosucc)
            if (.not. cosucc) then
                posqomega = oldposq; discreted = olddiscd; priceqh = oldpq; intermediatechi = oldchiq 
                y(j) = (y(j) + xin(j))/2.
                call spread1(y, fv(j+1), cosucc)
                print *, j, fv(j+1), cosucc
            end if
            if (pflag) print *, j, fv(j+1)
            v(1:n1, j+1) = y
        end do
        
	end subroutine	ctr_simplex01

! =========================================================================================
! =========================================================================================

	subroutine poly_loop01(func_evals, itercount, v, fv, how, success)
		use parameters_polytope1
		implicit none
	
		integer(4), intent(inout) :: func_evals, itercount
		real(8), dimension(n1, np1), intent(inout) :: v
		real(8), dimension(np1), intent(inout) :: fv
		character (len = 30), intent(out) :: how, success

		! internal variables
		integer(4), dimension(np1) :: permute	!permutation matrix
		real(8), dimension(n1) :: xbar, xe, xr, xc, xcc
		real(8) :: fxr, fxe, fxc, fxcc
		integer(4) :: j
        logical :: cosucc
        real(8) :: oldobj

        oldobj = fv(2)
        cosucc = .false.
        
        while_loop1: do

		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		if ((maxval(abs(fv(2:np1) - fv(1:n1)))<= tol_fun) .and. (maxval(abs(v(:, 2:np1) - v(:, 1:n1)))<= tol_par)) then
			success = "minimum found"
			exit while_loop1
		else if ((func_evals >= maxfun) .or. (itercount >= maxiter)) then
			success = "minimum not found"
			exit while_loop1
        end if
                
		! ================= END STOP COUNTROLS ========================

		xbar = sum(v(:, 1:n1), 2)/n1
		xr = (1 + prho)*xbar - prho*v(:, np1)
				
		call spread1(xr, fxr, cosucc)
		func_evals = func_evals + 1
		
		if1: if (fxr <= fv(1)) then
			! CALCULATE EXPANSION POINT
			xe = (1 + prho*pchi)*xbar - prho*pchi*v(:, np1)
					
			call spread1(xe, fxe, cosucc)
			func_evals = func_evals + 1
			if (fxe < fxr) then
				v(:, np1) = xe;
				fv(np1) = fxe;
				how = "expand"
			else
				v(:, np1) = xr;
				fv(np1) = fxr;
				how = "reflect"
			end if
		else	!(fv(1) <= fxr)
			if2: if (fxr < fv(n1)) then
				v(:, np1) = xr
				fv(np1) = fxr
				how = "reflect"
			else !if2: fxr >= fv(n1)
				
				! PERFORM CONTRACTION
				if31: if (fxr < fv(np1)) then
					! PERFORM OUTSIDE CONTRACTION
					xc = (1 + prho*ppsi)*xbar - prho*ppsi*v(:, np1)
							
					call spread1(xc, fxc, cosucc)
					func_evals = func_evals + 1
					if41: if (fxc <= fxr) then
						v(:, np1) = xc
						fv(np1) = fxc
						how = "contract outside"
					else !if41
						! PERFORM A SHRINK
						how = "shrink"
					end if if41
				else !if31
					! PERFORM INSIDE CONTRACTION
					xcc = (1 - ppsi)*xbar + ppsi*v(:, np1)
					
					call spread1(xcc, fxcc, cosucc)
					func_evals = func_evals + 1
					if42: if (fxcc < fv(np1)) then
						v(:, np1) = xcc
						fv(np1) = fxcc
						how = "contract inside"
					else !if42
						! PERFORM A SHRINK
						how = "shrink"
					end if if42
				end if if31
				if32: if (how == "shrink") then
					do j = 2, np1
						v(:, j) = v(:, 1) + psigma*(v(:, j) - v(:, 1))
						call spread1(v(:, j), fv(j), cosucc)
					end do 
					func_evals = func_evals + 1
				end if if32
			end if if2							
		end if if1
		! ================ SORT V AND FV ===========================
		permute = (/ (j, j=1,np1) /)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(n1, np1, v, n1, permute, two, v, n1)	!permutates the columns of v according to the sorting of fv
		! ================ END SORT V AND FV ========================
        if ((fv(1) < 0.3) .and. (cosucc)) exit while_loop1
		itercount = itercount + 1

        end do while_loop1

	end subroutine poly_loop01
