! ======================================================================
! ======================================================================
! sanneal.F90      minimizes a function using simulated annealing
! ======================================================================
! ======================================================================

subroutine sanneal(xinitial, xmin, fmin, smin)
	use parameters_sanneal
	use numerical_libraries 
	use parameters_model
	use parameters_quality
    implicit none
    
    real(8), dimension(n), intent(in) :: xinitial
    real(8), dimension(n), intent(out) :: xmin                  ! result, final minimizer
    real(8), intent(out) :: fmin                                ! function evaluated at minimizer
    logical, intent(out) :: smin                                ! success of function evaluated at minimizer

    ! simulated annealing variables
    real(8), dimension(n) :: xcurrent, xnew
    real(8) :: fcurrent, fnew
    logical :: scurrent, snew
    real(8) :: temp                                             ! current temperature
    integer(4) :: itry, itotal, isuccess, iconsec, ifail, j     ! counters
    real(8) :: rd
    
    ! equilibrium variables to be saved---specific to the Colombian project
    integer(4), dimension(sizef) :: posqmin, posqcurrent, posqnew
	integer(4), dimension(sizef) :: discmin, disccurrent, discnew			        	
    real(8), dimension(sizeq) :: pqmin, pqcurrent, pqnew
    real(8), dimension(sizeq) :: chiqmin, chiqcurrent, chiqnew

    ! body of sanneal
    ! ===================================================================
    xmin = xinitial
    call calc_123(xmin, fmin, smin) 
    call saveequil(posqmin, discmin, pqmin, chiqmin)
    if (.not. smin) then
        print *, 'equilibrium not found for initial guess of parameters'
    end if

    ! reset counters, temperature and initial values
    ! --------------------------------------
    temp = 300.d0
    itry = 1
    itotal = 0
    isuccess = 0
    iconsec = 0
    ifail = 0
    xcurrent = xmin
    fcurrent = fmin
    call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
    
    ! --------------------------------------
    sanneal_loop1: do
        ! stop commands
        if (ifail > maxfail) then
            print *, "no equilibrium found in a neighborhood of current"
            stop
        end if

        if ( (itry >= maxtry) .or. (isuccess >= maxsucc) ) then             ! change temperature or end program
            if ( (temp < stoptemp) .or. (iconsec >= maxconsec) ) then       ! stop program if temperature reaches a minimum or attempts at improving objective function failed
                itotal = itotal + itry
                print *, 'optimization algorithm finished in # tries', itotal
                call resetequil(posqmin, discmin, pqmin, chiqmin)
                call print_tsanneal(xmin, fmin)
                exit sanneal_loop1
            else
                call tcool(temp, temp)                                      ! cool temperature
                itotal = itotal + itry
                itry = 1
                isuccess = 0
                !print *, 'temperature cool', itotal, temp, fmin
                call resetequil(posqmin, discmin, pqmin, chiqmin)
                call print_tsanneal(xmin, fmin)
            end if
         end if

         ! draw new random
         call neighborx1(xcurrent, xnew)
         call resetequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
         
         call calc_123(xnew, fnew, snew)
         
         if (.not. snew) then       ! cycle if loop fails
            ifail = ifail + 1
            cycle sanneal_loop1
         else
            ifail = 0
            itry = itry + 1
         end if

         ! new minimum found: move current and save minimum
         if (fnew < fmin) then
            print *, 'd', temp, fmin, fnew
            xmin = xnew
            fmin = fnew
            smin = snew
            call saveequil(posqmin, discmin, pqmin, chiqmin)
            xcurrent = xnew
            fcurrent = fnew
            scurrent = snew
            call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
            iconsec = 0
            call print_tsanneal(xmin, fmin)
            cycle
          end if

          ! change parameters randomly
          call random_number(rd)
          if ( rd < exp( (fcurrent - fnew)/temp) ) then
            !print *, isuccess, fcurrent, fnew, exp((fcurrent - fnew)/temp)
            xcurrent = xnew
            fcurrent = fnew
            scurrent = snew
            call saveequil(posqcurrent, disccurrent, pqcurrent, chiqcurrent)
            isuccess = isuccess + 1
          else
            iconsec = iconsec + 1
          end if

    end do sanneal_loop1


    ! save results
    call resetequil(posqmin, discmin, pqmin, chiqmin)
    call calc_123(xmin, fmin, smin) 
    call print_sanneal(xmin, fmin)
    
end subroutine sanneal

! ======================================================================
! AUXILIARY SUBROUTINES
! ======================================================================

subroutine saveequil(posq0, disc0, pq0, chiq0)
! saves equilibrium values into posq0, disc0, pq0, chiq0
    use parameters_firm
    use parameters_quality
    implicit none

    integer(4), dimension(sizef), intent(out) :: posq0
	integer(4), dimension(sizef), intent(out) :: disc0
    real(8), dimension(sizeq), intent(out) :: pq0
    real(8), dimension(sizeq), intent(out) :: chiq0

    posq0 = posqomega
    disc0 = discreted
    pq0 = priceqh
    chiq0 = intermediatechi

end subroutine saveequil

! ======================================================================
subroutine resetequil(posq0, disc0, pq0, chiq0)
! resets the equilibrium values in posq0, disc0, pq0, chiq0 according to the ones in the last run program.
    use parameters_firm
    use parameters_quality
    implicit none

    integer(4), dimension(sizef), intent(in) :: posq0
	integer(4), dimension(sizef), intent(in) :: disc0
    real(8), dimension(sizeq), intent(in) :: pq0
    real(8), dimension(sizeq), intent(in) :: chiq0

    posqomega = posq0
    discreted = disc0
    priceqh = pq0
    intermediatechi = chiq0

end subroutine resetequil

! ======================================================================
subroutine tcool(tempin, tempout)
    implicit none 

    real(8), intent(in) :: tempin
    real(8), intent(out) :: tempout

    tempout = 0.8*tempin

end subroutine tcool

! ======================================================================
subroutine neighborx1(xcenter, xneighbor)
    use parameters_sanneal
    implicit none 

    real(8), dimension(n), intent(in) :: xcenter
    real(8), dimension(n), intent(out) :: xneighbor
    real(8) :: rd
    integer(4) :: j
    ! -------------------------------------------------------
    do j = 1, n
        call random_number(rd)
        xneighbor(j) = xcenter(j) + (rd - 0.5)*mydelta1(j)*2.
    end do
    
end subroutine neighborx1

! ======================================================================
subroutine  print_sanneal(x, obj)
    use parameters_quality
    use parameters_firm
    implicit none

	real(8), dimension(n), intent(in) :: x
    real(8), intent(in) :: obj
    integer(4) :: j
	real(8) :: aux
    logical, parameter :: auxl = .true.
    
	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
            
    open(1, FILE='result_sa1.txt', STATUS='REPLACE')
    write(1,*), obj
    write(1,*), auxl
	do j = 1,n
         write(1,*), x(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
	end do
	close(1)

end subroutine  print_sanneal


! ======================================================================
subroutine  print_tsanneal(x, obj)
    use parameters_quality
    use parameters_firm
    implicit none

	real(8), dimension(n), intent(in) :: x
    real(8), intent(in) :: obj
    integer(4) :: j
	real(8) :: aux
    logical, parameter :: auxl = .true.
    
	! ----------------------------------------------------
    ! record equilibrium posqomega, bigpq, chiq
            
    open(1, FILE='result_tsa1.txt', STATUS='REPLACE')
    write(1,*), obj
    write(1,*), auxl
	do j = 1,n
         write(1,*), x(j)
    end do
	do j = 1, sizef
		write(1,*), posqomega(j)
	end do
	do j = 1, sizef
		write(1,*), discreted(j)
	end do
	do j = 1, sizeq
		write(1,*), priceqh(j)
    end do
	do j = 1, sizeq
		write(1,*), intermediatechi(j)
	end do
	close(1)

end subroutine  print_tsanneal
