!*********************************************************************************************************
!
!  PROGRAM: MC_unique
!
!  PURPOSE: Monte Carlo simulations to check for uniqueness of equilibrium at parameter values
!
!*********************************************************************************************************

module parameters_model
		real(8), parameter :: mysigma = 5.0d0				                ! elasticity of substitution across sectors
        real(8), parameter :: markup = mysigma/(mysigma-1.d0)
        real(8), parameter :: myalpha = 0.7d0			            	    ! Cobb-Douglas weight on labor
        real(8), parameter :: tariff = .32d0	
        real(8), parameter :: income = 1.d0
        real(8), parameter :: sigmal = 1.6d0                                ! elasticity of substitution between skilled and unskilled workers
		integer(4), parameter :: sizef = 100000
        real(8), parameter :: womega = 1.d0/sizef                           ! firm weight (1/sizef)
        integer(4), parameter :: sizeq = 201					            ! size of q grid and number of firm draws
        real(8), parameter :: qlow = 0., qup = 10.d0                        ! lower and upper bound of existing varieties: nq varieties exist for any q\in [qlow,qbar]; low = (1.d0/(sizeq-1.))
        integer(4), parameter :: n = 18                                     ! number of parameters to be estimated. One extra is l3 which is calibrated (no standard errors)
        integer(4), parameter :: n1 = 13, n2 = 4, n3 = 1                    ! number of parameters to be estimated in each stage
        real(8) :: xminall(n), xmin1(n1), xmin2(n2), xmin3(n3)
        logical :: toprint, pflag
        real(8), parameter :: nfirms_data = 6710.d0                         ! number of plants (in March 8 moments)
        integer(4) :: nexit, nimport, nexport                               ! number of firms exiting, importing and exporting
end module parameters_model

	! ----------------------------------------------------------------------------
	 module parameters_quality
     use parameters_model
        real(8) :: pca, pch, pcf                                        ! consumer's price indices---total, domestic and foreign only
	    real(8), dimension(sizeq) :: qgrid
	    real(8), dimension(sizeq) :: chiq, chiqf                        ! demand shifter in Home and Foreign
        real(8), dimension(sizeq) :: priceqa, priceqh, priceqf          ! price index all goods, home and foreign goods only
		real(8), dimension(sizeq) :: consumptionchi, intermediatechi
		real(8), dimension(sizeq, 4) :: rfirmp                          ! price component for each firm type; representative firm p
		real(8), dimension(sizeq, 4) :: nq, zq                  		! zq=1/aq is the productivity of a representative firm (in the Melitz sense) of quality q.
		real(8), dimension(sizeq) :: fxq	    	                    ! fixed cost of production for each firm-quality
        real(8), parameter, dimension(sizeq) :: wq=1.d0                      ! labor cost index for firm of quality q
        
        real(8), dimension(sizeq) :: chiq0
        real(8), dimension(sizeq) :: priceqh0
        
     end module parameters_quality
     
	! ----------------------------------------------------------------------------
	module parameters_firm
        use parameters_model
		integer(4), dimension(sizef) :: posqomega			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
        integer(4), dimension(sizef) :: posqomega0			        	! positition of quality choice
		integer(4), dimension(sizef) :: discreted0			        	! discrete decisions: discreted = 0 if exit, 1 if no trade, 2 if imports only, 3 if exports only and 4 if imports and exports
	end module parameters_firm

	! ----------------------------------------------------------------------------
	module parameters_qf
		use parameters_model
		real(8), dimension(sizef) :: qomega						        ! choice of quality (level)
		real(8), dimension(sizef, sizeq) :: zqomega		        		! productivity of each firm 
        real(8), dimension(sizef) :: fxomega, fmomega                   ! firm-specific import and export cost
        real(8), dimension(sizef,2) :: skomega                          ! firm-specific measurement error of skilled workers
    end module parameters_qf
    
    ! ----------------------------------------------------------------------------
	module random_draw
	    use parameters_model
        real(8), dimension(sizef,2) :: random_firms
        real(8), dimension(sizef,2) :: random_fxcost
        real(8), dimension(sizef) :: random_skill
    end module random_draw  
    
 ! ----------------------------------------------------------------------------
    module parameters_estimate
		real(8) :: mu1, sigma1                                          ! z1(omega) log normal mean and variance
        real(8) :: sigma2                                               ! z2(omega) normal, variance. Mean is zero.
        real(8) :: f1, f2                                               ! fixed cost parameters
        real(8) :: fm, fx                                               ! fixed costs to import and export
        real(8) :: fmvar, fxvar                                         ! variance of fixed import and export cost
        real(8) :: qf                                                   ! quality of representative foreign firm 
        real(8) :: qcf                                                  ! quality of representative foreign consumer
        real(8) :: yf 
        real(8) :: nu                                                   ! complementarity parameter in production
        real(8), parameter :: pf=1.d0
    end module parameters_estimate

    ! =============================================================================================
    program MC_unique
    ! =============================================================================================
    INCLUDE 'link_fnl_static.h'
    use numerical_libraries
    use parameters_estimate
    use parameters_quality
    use parameters_firm
        
    implicit none
    
    ! Variables
    ! =============================================================================================
   	real(8), dimension(n) :: xinitial
    real(8) :: xinit1(n1)
    logical :: succ
    integer(4), parameter :: maxguess = 1000          ! repeat initial draws of posqomega, discreted
    integer(4) :: nguess, nfail, ndiffq, ndiffd, ndiffq0, ndiffd0
    integer(4) :: j
    integer :: startime, endtime, hz
    real(8) :: rdd, rdq
	
    ! results from optimization algorithms
    real(8), dimension(n) :: xmin
    logical :: smin                                 ! success
    
    ! =============================================================================================
    ! preliminaries: get weights, get random variables, construct grid
    ! ----------------------------------------------------------------
    call read_random
    call gen_qgrid
    
    ! get parameters, firm characteristics and foreign variables that do not depend on domestic firm choices. 
    ! ----------------------------------------------------------------
    intermediatechi = 1.d0; priceqh = 1.d0
    call read_initial(xinitial)
    call xtoguess(xinitial)
    call print_parameters
    call gen_xfirms          ! firm productivity, fixed import and export cost
    call gen_xquality
    
    ! equilibrium at initial guess of posqomega, discreted (from result1s)
    ! ----------------------------------------------------------------
    toprint =  .false.
    succ = .false.
    call equilibrium(succ)
    posqomega0 = posqomega
    discreted0 = discreted
    chiq0 = chiq
    priceqh0 = priceqh
    if (.not. succ) then; print *, 'equilibrium not found for initial guess, _MC_unique.f90'; stop; else; print *, 'successful initial guess'; print *, ' '; end if
    
    ! ----------------------------------------------------------
	open(22, FILE='MC1_results.dat', STATUS='REPLACE')	    
    open(23, FILE='MC1_results.csv', STATUS='REPLACE')	  
    open(24, FILE='MC1_dPdchi.csv', STATUS='REPLACE')	  
    open(25, FILE='MC1_dPdchi2.csv', STATUS='REPLACE')	  
    write(23,*), 'nguess, succ, ddisc0, dpos0'
    write(24,*), 'nguess, dph, dchi'    ! (1-0)
    write(25,*), 'nguess, dph2, dchi2'  ! (1-0)/0
    
    
    ! iterate over random guesses of firm choices
    ! ----------------------------------------------------------------------
    nguess = 1
    nfail = 0
    ndiffd = 0
    ndiffq = 0
    
    print *, '   nguess, succ,    initial diff d q,    final diff d q'
    write(22,*), '   nguess, succ,    initial diff d q,    final diff d q'
    
    do while (nguess <= maxguess)
        
        ! --------------------------------------------------------------
        ! get new random guess
        do j = 1, sizef
            call RANDOM_NUMBER(rdd)
            if (rdd>0.9d0) then  ! firm exits
                discreted(j) = 0
                posqomega(j) = -1
            else    
                discreted(j) = int(4*rdd/0.9d0) + 1
                call RANDOM_NUMBER(rdq)
                posqomega(j) = int(rdq*(sizeq-1)) + 1
            end if
        end do
        
        ! assert choices are within bounds
        where (discreted==0) posqomega = -1
        where ((discreted>0) .and. ((posqomega<-1) .or. (posqomega>sizeq))) posqomega = sizeq/2
        ndiffd0 = count(discreted0 /= discreted)
        ndiffq0 = count(posqomega0 /= posqomega)
                
        ! find equilibrium
        call equilibrium(succ)
        
        ! record results
        if (succ) then
            print *, nguess, succ, ndiffd0, ndiffq0, count(discreted0 /= discreted), count(posqomega0 /= posqomega)
            write(22,*), nguess, succ, ndiffd0, ndiffq0, count(discreted0 /= discreted), count(posqomega0 /= posqomega)
            write(23,*), nguess, ', ', succ, ', ', count(discreted0 /= discreted), ', ', count(posqomega0 /= posqomega)
            write(24,*), nguess, ', ', sum(abs(priceqh0 - priceqh))/sizeq, ', ', sum(abs(chiq0 - chiq))/sizeq
            write(25,*), nguess, ', ', sum(abs(priceqh0 - priceqh)/priceqh0)/sizeq, ', ', sum(abs(chiq0 - chiq)/chiq0)/sizeq
            ndiffd = ndiffd + count(discreted0 /= discreted)
            ndiffq = ndiffq + count(posqomega0 /= posqomega)
            nguess = nguess + 1
        else
            print *, 'attempt unsuccessful', nguess, nfail
            nfail = nfail + 1
        end if
    end do
    print *, ' '
    print *, 'total failures', nfail
    print *, 'total diff d  ', ndiffd, real(ndiffd)/(sizef*maxguess)
    print *, 'total diff q  ', ndiffq, real(ndiffq)/(sizef*maxguess)
    
    write(22,*), ' '
    write(22,*), 'equilibrium not found', nfail
    write(22,*), 'total diff d  ', ndiffd, real(ndiffd)/(sizef*maxguess)
    write(22,*), 'total diff q  ', ndiffq, real(ndiffq)/(sizef*maxguess)
    close(22)
    
    end program MC_unique