! ***********************************************************************************************
! contains auxiliary subroutines to find equilibrium
!
! nqzq : finds representative firm's mass and productivity
! prices: finds price indices
! chiq: demand function chi
! ***********************************************************************************************


! ================================================================================================
subroutine  calc_nqzqd
	use parameters_quality
    use parameters_estimate
    use parameters_firm
    use parameters_qf
    implicit none

	integer(4) :: j, jj
	! ----------------------------------------------------

    nq = 0.d0
    zq = 0.d0
    do j = 1, sizef
        if (discreted(j)>0) then
            nq(posqomega(j),discreted(j)) = nq(posqomega(j),discreted(j)) + womega
	        zq(posqomega(j),discreted(j)) = zq(posqomega(j),discreted(j)) + (zqomega(j, posqomega(j))**(mysigma-1.))*womega
 	    end if
    end do
    where(nq>0.d0) zq = (zq/nq)**(1./(mysigma-1.))
    
    if ( any(isnan(zq)) .or. any(isnan(zq)) .or. any(isnan(zq*0.))  .or. any(isnan(nq*0.)) ) then
        print *, 'zq, nq is Nan in auxequilibrium.f90, calc_nqzqd'
        stop
    end if
        
end subroutine  calc_nqzqd


! ================================================================================================
subroutine prices
    use parameters_quality
    use parameters_estimate
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolp = 1.e-15
	integer(4), parameter :: maxiter = 500
	integer(4) :: iter, j, outq
	real(8) :: error
	real(8), dimension(sizeq) :: oldpq, qvdiff, phivo
	! ----------------------------------------------------
    
    iter = 1
    error = 10.*tolp
    
    ! firm prices
    ! =======================================================================================
    price_loop: do
        ! stop commands --------------------------------------
        if (error <= tolp) then
            !print *, 'prices converged', iter
            exit price_loop
	    elseif (iter >= maxiter) then
    		print *, "prices did not converge ********************", error
	    	!stop
        end if
        ! ----------------------------------------------------
        
        ! calculate auxiliary prices from new price guess
        priceqa = ( priceqf**(1.-mysigma) + priceqh**(1-mysigma) )**(1./(1.-mysigma))
        rfirmp(:,1) = markup*priceqh**(1.-myalpha)*wq**myalpha/zq(:,1)
        rfirmp(:,3) = markup*priceqh**(1.-myalpha)*wq**myalpha/zq(:,3)
        rfirmp(:,2) = markup*priceqa**(1.-myalpha)*wq**myalpha/zq(:,2)
        rfirmp(:,4) = markup*priceqa**(1.-myalpha)*wq**myalpha/zq(:,4)
        
        ! calculate new priceqh
        oldpq = priceqh
        outq_loop: do outq = 1, sizeq
            qvdiff = qgrid - nu*qgrid(outq)
            call phiv0(sizeq, qvdiff, phivo)
            priceqh(outq) = ( sum ( sum( rfirmp**(1.-mysigma)*nq ,2) * phivo ) )**(1./(1.-mysigma))
        end do outq_loop
    
        ! get errors
        error = sum(abs(oldpq - priceqh))
        iter = iter + 1
    end do price_loop
    
    ! use final price to recalculate auxiliary prices and consumer prices
    priceqa = ( priceqf**(1.-mysigma) + priceqh**(1-mysigma) )**(1./(1.-mysigma))
    rfirmp(:,1) = markup*priceqh**(1.-myalpha)*wq**myalpha/zq(:,1)
    rfirmp(:,3) = markup*priceqh**(1.-myalpha)*wq**myalpha/zq(:,3)
    rfirmp(:,2) = markup*priceqa**(1.-myalpha)*wq**myalpha/zq(:,2)
    rfirmp(:,4) = markup*priceqa**(1.-myalpha)*wq**myalpha/zq(:,4)
    
    ! consumer prices
    ! =======================================================================================
    call phiv0(sizeq, qgrid, phivo)
    pch = ( sum ( sum( rfirmp**(1.-mysigma)*nq ,2) * phivo ) )**(1./(1.-mysigma))
    pca = ( pch**(1.-mysigma) + pcf**(1.-mysigma) )**(1./(1.-mysigma))
       
end subroutine prices


! ================================================================================================
! recalculates prices using firm data and compares it to prices calculated from nq, zq above
subroutine check_prices
    use parameters_quality
    use parameters_qf
    use parameters_firm
    use parameters_estimate    
    implicit none
    
    integer(4) :: j
    real(8) :: firmpt
	real(8), dimension(sizeq) :: qvdiff, phivo, ptest
    real(8) :: qrdiff, pct, phiro
	! ----------------------------------------------------

    ptest = 0.d0
    pct = 0.d0
    do j = 1, sizef
        if (discreted(j) > 0) then
            ! calculate firm shifters phi
            qvdiff = qgrid(posqomega(j)) - nu*qgrid
            call phiv0(sizeq, qvdiff, phivo)
            
            ! calculate firm price
            if ( (discreted(j) == 1) .or. (discreted(j) == 3) ) then
                firmpt = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            else
                firmpt = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            end if
            ptest = ptest + womega*firmpt**(1.-mysigma)*phivo
            
            ! calculate consumer price
            qrdiff = qgrid(posqomega(j))
            call phir0(qrdiff, phiro)
            pct = pct + womega*firmpt**(1.-mysigma)*phiro
            
        end if
    end do
    ptest = ptest**(1./(1.-mysigma))
    pct = pct**(1./(1.-mysigma))
    
    ! comparison with prices
    if(toprint) then
        print *, 'error from check prices, abs, %'
        print *, 'firms    ', maxval(abs(ptest - priceqh)), maxval(abs(ptest - priceqh)/priceqh)
        print *, 'consumers', abs(pct - pch), abs(pct - pch)/pch
    end if
    
    if ( maxval(abs(ptest - priceqh)) + maxval(abs(ptest - priceqh)/priceqh) + abs(pct - pch) + abs(pct - pch)/pch > 1.e-3) then
        print *, 'checkprices: error in prices is too large, auxequilibrium.f90', maxval(abs(ptest - priceqh)) + maxval(abs(ptest - priceqh)/priceqh) + abs(pct - pch) + abs(pct - pch)/pch 
       ! stop
    end if
    
end subroutine check_prices



! ================================================================================================
subroutine calc_chiq
    use parameters_quality
    use parameters_estimate
    implicit none
    
    ! ----------------------------------------------------
    real(8), parameter :: tolchi = 1.e-10
	integer(4), parameter :: maxiter = 1000
	integer(4) :: iter, j, outq
	real(8) :: error, rd
	real(8), dimension(sizeq) :: oldchi, qvdiff, phivo
    real(8), dimension(sizeq, 4) :: rfirmaux
	
    ! consumption
    ! ----------------------------------------------------
    call phiv0(sizeq, qgrid, phivo)
    consumptionchi = phivo*pca**(mysigma-1.d0)*income
    
    ! intermediates
    ! ================================================================
    iter = 1
    error = 100.*tolchi
    
    chi_loop : do
        !stop commands
    	! --------------------------------
    	if (error <= tolchi) then
            !print *, 'chi converged   ', iter
    		exit chi_loop
    	elseif ( (iter >= maxiter) .or. (isnan(error)) .or. (any(isnan(intermediatechi/intermediatechi) .or. (isnan(intermediatechi)) )) ) then
    		print *, "chi did not converge", iter, error
            stop
        end if
        oldchi = intermediatechi   
    
        ! calculate total chi and rfirmaux = P^(sigma-1) * rt(representative firm) * weightnq * constant
        chiq = consumptionchi + intermediatechi
        rfirmaux(:,1) = nq(:,1) * rfirmp(:,1)**(1.-mysigma) * chiq * priceqh**(mysigma-1.)
        rfirmaux(:,2) = nq(:,2) * rfirmp(:,2)**(1.-mysigma) * chiq * priceqa**(mysigma-1.)
        rfirmaux(:,3) = nq(:,3) * rfirmp(:,3)**(1.-mysigma) * (chiq + chiqf) * priceqh**(mysigma-1.)
        rfirmaux(:,4) = nq(:,4) * rfirmp(:,4)**(1.-mysigma) * (chiq + chiqf) * priceqa**(mysigma-1.)
        rfirmaux = rfirmaux*(1.-myalpha)/markup
        
        ! new guess for chiq
        outq_loop2: do outq = 1, sizeq
            qvdiff = qgrid(outq) - nu*qgrid
            call phiv0(sizeq, qvdiff, phivo)
            intermediatechi(outq) = ( sum ( sum( rfirmaux,2 ) * phivo ) )
        end do outq_loop2
    
        if (error>50.) then
            call random_number(rd) 
            intermediatechi = (0.7 + rd*0.3)*intermediatechi + (0.3 - rd*0.3)*oldchi 
        end if
                        
        error = maxval(abs(intermediatechi - oldchi)/intermediatechi) + maxval(abs(intermediatechi - oldchi))
        iter = iter + 1
    end do chi_loop
    
    chiq = consumptionchi + intermediatechi  
    
    
end subroutine calc_chiq


! ================================================================================================
! recalculates prices using firm data and compares it to prices calculated from nq, zq above
subroutine check_chi
    use parameters_quality
    use parameters_qf
    use parameters_firm
    use parameters_estimate    
    implicit none
    
    integer(4) :: j
    real(8) :: firmpt, fraux
	real(8), dimension(sizeq) :: qvdiff, phivo, ichit
	! ----------------------------------------------------

    ichit = 0.d0
    
    do j = 1, sizef
        select case(discreted(j))
        case(1)
            firmpt = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            fraux = priceqh(posqomega(j))**(mysigma-1.) * firmpt**(1.-mysigma) * chiq(posqomega(j))
        case(2)
            firmpt = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            fraux = priceqa(posqomega(j))**(mysigma-1.) * firmpt**(1.-mysigma) * chiq(posqomega(j))
        case(3)
            firmpt = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            fraux = priceqh(posqomega(j))**(mysigma-1.) * firmpt**(1.-mysigma) * ( chiq(posqomega(j)) + chiqf(posqomega(j)) )
        case(4)
            firmpt = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            fraux = priceqa(posqomega(j))**(mysigma-1.) * firmpt**(1.-mysigma) * ( chiq(posqomega(j)) + chiqf(posqomega(j)) )
        end select
            
        ! calculate firm shifters phi
        if (discreted(j) > 0) then
            qvdiff = qgrid - nu*qgrid(posqomega(j))
            call phiv0(sizeq, qvdiff, phivo)
            ichit = ichit + phivo*fraux
        end if           
    end do
    ichit = ichit*(1.-myalpha)/markup*womega
    
    ! comparison with intermediatechi
    if(toprint) then
        print *, 'error from check chi, abs, %', maxval(abs(ichit - intermediatechi)), maxval(abs(ichit - intermediatechi)/intermediatechi)
    end if
    
    if ( maxval(abs(ichit - intermediatechi)) + maxval(abs(ichit - intermediatechi)/intermediatechi) > 1.e-3) then
        print *, 'check_chi: error is too large, auxequilibrium.f90', maxval(abs(ichit - intermediatechi)) + maxval(abs(ichit - intermediatechi)/intermediatechi) 
        !stop
    end if
    
end subroutine check_chi