! PURPOSE: to calculate out-of-sample moments on prices and skills
! ===========================================================================
subroutine outofsample
    use parameters_estimate
    use parameters_quality
    use parameters_firm
    use crosssec
    use data_moments
    implicit none
    
    ! normalized total sales, employment, average wage and khandelwal's quality measure
    real(8), dimension(sizef) :: employment, avgwage, kmeasure        
    integer(4), dimension(sizef) :: qsize
    real(8), dimension(sizef) :: auxv, auxv2
    real(8), dimension(4) :: ksize_model        ! average khandelwal measure by quartile of firm size
    real(8), dimension(4), parameter :: ksize_data = (/-0.585, -0.068, 0.466, 1.042/)       ! k_imp_ai
    real(8) :: aggskat, aggskmt
    integer(4) :: j, jj
    real(8) :: imps_model, exps_model
    real(8) :: firm_imp, cimp
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector, yvector2
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: incoeff, outcoeff       ! regression results: coefficients
    real(8), dimension(nvar, nvar) :: invar, outvar
    
    ! import patterns
    integer(4), parameter :: nivar = 3
    real(8), dimension(nivar) :: dicoeff, sicoeff       ! regression results: coefficients
    real(8), dimension(nivar, nivar) :: divar, sivar
    
    ! statistics of kmeasure (2 uses only domestic sales)
    real(8) :: sek1, sek2, corrk
    
    real(8) :: pkmeasure_model(7)
    
    print *, ' '
    print *, '---------------------------------------------------------'
    print *, 'OUT OF SAMPLE MOMENTS'
    print *, '---------------------------------------------------------'
    
    ! aggregate trade shares
    ! --------------------------------------------------------
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    imps_model = (cimp + firm_imp)/absorption/(1.+tariff)
    exps_model = sum(exp_share*tsales*womega)/absorption
    print*, 'international trade/absorption, data, model'
    print*, 'imports', imps_data, imps_model
    print*, 'exports', exps_data, exps_model
    print *, 'tariff, gross imp.', tariff, (cimp + firm_imp)/absorption
    print *, ' '
    print *, '---------------------------------------------------------'
    
    ! aggregate skill intensity and skill premium
    ! --------------------------------------------------------
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment = (myalpha/markup)*tsales/avgwage
    end where
    aggska = sum(employment*skilla)/sum(employment)
    
    ! get truncated aggregate skills
    !call taggska(tsales, employment, aggskat, aggskmt)
    
    print *, 'aggregate skills' ! new baseline, jan 27 in february files
    print *, ' '
    print *, 'measured, data (avg 1982-88), model' 
    print *, 'intensity', skshare_data, aggskm !, aggskmt
    print *, 'premium  ', skpremium_data, aggskpm
    print *, ' '
    print *, 'actual, data (avg 1982-88, Attanasio et al.), model' 
    print *, 'intensity    0.085', aggska !, aggskat
    print *, 'premium    1.8-2.6', skpremium
    print *, ' '
    print *, '---------------------------------------------------------'
    
    !price regressions & khandelwal measures
    ! -----------------------------------------------------------------
    where (discreted == 0)
        kmeasure = 0.d0
        auxv = 0.d0
        auxv2 = 0.d0
    elsewhere
        kmeasure = log(tsales*(1.-exp_share)) - (1.-mysigma)*log(outprice)
        auxv = log(tsales) - (1.-mysigma)*log(outprice)
        auxv2 = log(chiq(posqomega)) + (1.-mysigma)*log(zbarq(posqomega))
    end where
    kmeasure = kmeasure - sum(kmeasure)/(sizef - nexit)     ! demean
    auxv = auxv - sum(auxv)/(sizef-nexit)
    auxv2 = auxv2 - sum(auxv2)/(sizef-nexit)
    where (discreted == 0)
        kmeasure = 0.d0
        auxv = 0.d0
        auxv2 = 0.d0
    end where
        
    
    ! unconditional distribution of k-measures
    call pdistribution2(kmeasure, pkmeasure_model)   
    
    print *, ' ----------------------------------------------------- '
    print *, 'distribution of kmeasures, model'
    print *, ' 5% ', pkimp_data(1), pkmeasure_model(1)
    print *, '10% ', pkimp_data(2), pkmeasure_model(2)
    print *, '25% ', pkimp_data(3), pkmeasure_model(3)
    print *, '50% ', pkimp_data(4), pkmeasure_model(4)
    print *, '75% ', pkimp_data(5), pkmeasure_model(5)
    print *, '90% ', pkimp_data(6), pkmeasure_model(6)
    print *, '95% ', pkimp_data(7), pkmeasure_model(7)
    print *, 'test = 0?', sum(abs(kmeasure - auxv2)), sum(kmeasure)
    
    ! standard errors, correlation between kmeasure using all sales and domestic sales only
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs))
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = kmeasure(j)
            yvector2(jj) = auxv(j)
            !print *, j, kmeasure(j), auxv(j)
        end if
    end do
    call correlate12(tobs, yvector, yvector2, sek1, sek2, corrk)
    
    deallocate(yvector, yvector2)
    print *, 'standard deviation of k-measure, total, domestic sales', sek1, sek2
    print *, 'correlation', corrk
    print *, ' '
    print *, " ---------------------------------------- "
    
    ! khandelwal measure by quartile of firm domestic sales
    call quartiles(log(nhsales), qsize)
    do j = 1, 4
        auxv = 0.d0
        where (qsize == j) auxv = 1.d0
        ksize_model(j) = sum( auxv*kmeasure )/sum(auxv)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    print *, 'khandelwal measure of quality by quartiles of size (model only, data in excell)'
    print *, '1 ', ksize_model(1)
    print *, '2 ', ksize_model(2)
    print *, '3 ', ksize_model(3)
    print *, '4 ', ksize_model(4)
    print *, ' '
    print *, " ---------------------------------------- "
    print *, 'price regressions' ! new baseline, jan 27 in february files
    print *, ' '
    tobs = sizef - nexit
    allocate(yvector(tobs), yvector2(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! measured skill intensity (with standard errors)
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            yvector2(jj) = log(inprice(j))
            xmat(jj,2) = skillm(j)
        end if
    end do
    yvector = yvector - sum(yvector)/real(tobs)     ! demean price vectors
    yvector2 = yvector2 - sum(yvector2)/real(tobs)
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    call ols(tobs, nvar, yvector2, xmat, incoeff, invar) 
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (skill intensity)'
        print*, 'output coef', skill_pdata(1), outcoeff(2)
        print*, 'output se  ', skill_sepdata(1), outvar(2,2)**0.5
        print*, 'input coeff', skill_pdata(2), incoeff(2) 
        print*, 'input se  ', skill_sepdata(2), invar(2,2)**0.5
        print *, ' '
    end if
    
    ! ln sales
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            xmat(jj,2) = log(tsales(j))
        end if
    end do
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    call ols(tobs, nvar, yvector2, xmat, incoeff, invar) 
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on ln(sales)'
        print*, 'output coef', sales_pdata(1), outcoeff(2)
        print*, 'output se  ', sales_sepdata(1), outvar(2,2)**0.5
        print*, 'input coeff', sales_pdata(2), incoeff(2) 
        print*, 'input se  ', sales_sepdata(2), invar(2,2)**0.5
        print *, ' '
    end if
    
    ! ---------------------------------------------------------------------
    ! khandelwal measure
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            xmat(jj,2) = kmeasure(j)
        end if
    end do
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    call ols(tobs, nvar, yvector2, xmat, incoeff, invar) 
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on k-measures'
        print*, 'output coef', kimpai_pdata(1), outcoeff(2)
        print*, 'output se  ', kimpai_sepdata(1), outvar(2,2)**0.5
        print*, 'input coeff', kimpai_pdata(2), incoeff(2) 
        print*, 'input se  ', kimpai_sepdata(2), invar(2,2)**0.5
        print *, ' '
    end if  
    
    ! -----------------------------------------------------
    ! importing patterns
    ! -----------------------------------------------------
    print *, " ---------------------------------------- "
    print *, 'importing patterns' ! new baseline, jan 27 in february files
    print *, ' '
    
    ! regression of import dummy
    ! ---------------------------------------------------------------------
    deallocate(yvector, yvector2, xmat)
    tobs = sizef - nexit
    allocate(yvector(tobs), xmat(tobs,nivar))
    xmat(:,1) = 1.d0
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            if ( (discreted(j)==2) .or. (discreted(j)==4)) then
                yvector(jj) = 1.d0
            else
                yvector(jj) = 0.d0
            end if
            xmat(jj,2) = skillm(j)
            xmat(jj,3) = log(tsales(j))
        end if
    end do
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nivar, yvector, xmat, dicoeff, divar)      ! output prices on skill intensity
    
    
    ! regression of import share
    ! ---------------------------------------------------------------------
    deallocate(yvector, xmat)
    tobs = nimport
    allocate(yvector(tobs), xmat(tobs,nivar))
    xmat(:,1) = 1.d0
    jj = 0
    do j = 1, sizef
        if( (discreted(j)==2) .or. (discreted(j)==4)) then
            jj = jj + 1
            yvector(jj) = log(imp_share(j))
            xmat(jj,2) = skillm(j)
            xmat(jj,3) = log(tsales(j))
        end if
    end do
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nivar, yvector, xmat, sicoeff, sivar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'regression of import dummy on skill intensity and sales'
        print*, 'skill coef', 0.099d0, dicoeff(2)
        print*, 'skill se  ', 0.010d0, divar(2,2)**0.5
        print*, 'n obs.     46,770', sizef - nexit
        print*, 'regression of import shares on skill intensity and sales'
        print*, 'skill coef', 0.144, sicoeff(2)
        print*, 'skill se  ', 0.012, sivar(2,2)**0.5
        print*, 'n obs.     12,041', nimport
        print *, ' '
    end if

    ! partial equilibrium response to a change in import tariffs
    ! ========================================================================
    !call partial
     
    ! market shares, fixed costs
    ! patterns for material purchases, from skill to skill firms, need to be activated (too much memory space)
    ! ========================================================================
    
    print *, 'to show market shares and fixed costs, activate subroutine in outofsample.f90'
    !call mktshares
        
end subroutine outofsample

! ========================================================================
! estimate partial equilibrium response to a change in import tariffs
! ========================================================================
subroutine  partial
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use crosssec
    use random_draw
	implicit none

	integer(4), dimension(sizef) :: posqomega2, discreted2
    
    real(8), dimension(sizeq, 4) :: profit
    integer, dimension(2) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8) :: fprofit
    real(8), dimension(sizeq) :: newpqf, newpqa
    real(8), dimension(sizef) :: outprice2, skilla2, skillm2, imp_share2, exp_share2, mwage, mwage2, tsales2, auxv, kmeasure2
    real(8) :: Fu, Fl, pil
    real(8), parameter:: dtariff = -0.1d0    !may17I
    !real(8), parameter:: dtariff = -0.13d0     !mar17W
    
    ! -------------------------------------------------------------------
    ! change in import tariffs
    newpqf = priceqf*(1.d0+dtariff)
    newpqa = (priceqh**(1.-mysigma) + newpqf**(1.-mysigma))**(1./(1.-mysigma))
    
   ! --------------------------------------------------------------------    
    !$OMP PARALLEL shared(posqomega2, discreted2), private(profit, pos_maxpi, fprofit)
    !$OMP do
    do j = 1, sizef
        ! calculate profit for firm j
         profit(:, 1) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq
         profit(:, 2) = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq - fmomega(j)
         profit(:, 3) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - fxomega(j)
         profit(:, 4) = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - (fmomega(j) + fxomega(j))
         
         ! update its choice
         fprofit = maxval(profit)
         pos_maxpi = maxloc(profit)
         if (fprofit >=0.d0) then
            posqomega2(j) = pos_maxpi(1)
            discreted2(j) = pos_maxpi(2)
         else
            posqomega2(j) = -1
            discreted2(j) = 0
         end if
         
    end do
    !$OMP END do
    !$OMP END PARALLEL
    
    ! calculate new import shares, skills, etc
    ! ----------------------------------------------------------------------
    Fu = exp(1./varpi)/(1. + exp(1./varpi))        ! truncation of logistic
    Fl = 0.5d0 
    
    do j = 1, sizef
		select case(discreted2(j))
        case(0)
            outprice2(j) = 0.d0
            tsales2(j) = 0.d0
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
            skilla2(j) = 0.d0
            skillm2(j) = 0.d0
        case(1)
            outprice2(j) = markup*priceqh(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
        case(2)
            outprice2(j) = markup*newpqa(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            exp_share2(j) = 0.
            imp_share2(j) = ( newpqf(posqomega2(j))/newpqa(posqomega2(j)) )**(1.-mysigma)
        case(3)
            outprice2(j) = markup*priceqh(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = 0.
        case(4)
            outprice2(j) = markup*newpqa(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = ( newpqf(posqomega2(j))/newpqa(posqomega2(j)) )**(1.-mysigma)
        end select
        
        if (discreted(j) > 0) then
            outprice2(j) = outprice2(j)/zbarq(posqomega2(j))
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla2(j) = skillq(posqomega2(j))
            Fu = 1. - skilla2(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm2(j) = skillq(posqomega2(j)) + pil
            if (isnan(skillm2(j) )) then
                skillm2(j) = 1.
                pil = 1./skilla2(j) - 1.d0
            end if            
        end if
    end do
    !tsales2 = (womega*tsales2)/absorption
    
    ! compare to original choices 
    ! -------------------------------------------------------
    print *, 'partial equilibrium responses to tariff change of', dtariff
    print *, ' ---------------------------------------------------- '
    
    ! entry
    print *, 'entry', count((discreted==0) .and. (discreted2>0))/real(sizef - nexit)
    
    ! import dummy
    print *, 'import dummy', count( ((discreted==1) .or. (discreted==3)) .and. ((discreted2==2) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! import intensity
    print *, 'import intensity', sum(imp_share2 - imp_share)/(sizef - nexit) 
    
    ! export dummy
    print *, 'export dummy', count( ((discreted==1) .or. (discreted==2)) .and. ((discreted2==3) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! export intensity
    print *, 'export intensity', sum(exp_share2 - exp_share)/(sizef - nexit) 
    
    ! skill share
    print *, 'average increase in skill share (actual)', sum(skilla2 - skilla)/(sizef - nexit)
    print *, 'average increase in skill share (measur)', sum(skillm2 - skillm)/(sizef - nexit)
    
    ! wages
    mwage = skilla*skpremium + (1.-skilla)
    mwage2 = skilla2*skpremium + (1.-skilla2)
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(mwage2) - log(mwage)
    elsewhere
        auxv = 0.d0
    end where
    print *, 'average increase in wage', sum(auxv)/(sizef - nexit) 
        
    ! output prices
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(outprice2) - log(outprice)
    elsewhere
        auxv = 0.d0
    end where    
    print *, 'average increase in output price', sum(auxv)/(sizef - nexit) 
        
    auxv = 0.d0
    do j = 1, sizef
        if ((discreted(j)>0) .and. (discreted2(j)>0)) auxv(j) = (1. - imp_share(j))*inpq(posqomega2(j))/inpq(posqomega(j)) + imp_share(j) - 1.d0
    end do
    print *, 'input price domestic only', sum(auxv)/(sizef - nexit) 
    
    ! kmeasures
    kmeasure2 = 0.d0
    do j = 1, sizef
        if ((discreted(j)>0) .and. (discreted2(j)>0)) kmeasure2(j) = log(chiq(posqomega2(j))) + (1.-mysigma)*log(zbarq(posqomega2(j))) - log(chiq(posqomega(j))) - (1.-mysigma)*log(zbarq(posqomega(j)))
    end do
    
    print *, 'k-measure', sum(kmeasure2)/(sizef - nexit) 
    print *, ' '
    
end subroutine partial


! ========================================================================
! estimate partial equilibrium response to a change in import tariffs, when firms cannot change discrete choices
! ========================================================================
subroutine  partial2
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use crosssec
    use random_draw
	implicit none

	integer(4), dimension(sizef) :: posqomega2, discreted2
    
    real(8), dimension(sizeq) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8) :: fprofit
    real(8), dimension(sizeq) :: newpqf, newpqa
    real(8), dimension(sizef) :: outprice2, skilla2, skillm2, imp_share2, exp_share2, mwage, mwage2, tsales2, auxv, kmeasure2
    real(8) :: Fu, Fl, pil
    real(8), parameter:: dtariff = -0.001d0
    
    ! -------------------------------------------------------------------
    ! change in import tariffs
    newpqf = priceqf*(1.d0+dtariff)
    newpqa = (priceqh**(1.-mysigma) + newpqf**(1.-mysigma))**(1./(1.-mysigma))
    discreted2 = discreted
   ! --------------------------------------------------------------------    
    !$OMP PARALLEL shared(posqomega2, discreted2), private(profit, pos_maxpi, fprofit)
    !$OMP do
    do j = 1, sizef
        if ((discreted(j) <=1) .or. (discreted(j) == 3)) then
            posqomega2(j) = posqomega(j)
        else if (discreted(j) == 2) then
            ! calculate profit for firm j
            profit = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq - fmomega(j)
            pos_maxpi = maxloc(profit)
            posqomega2(j) = pos_maxpi(1)
        else if (discreted(j) == 4) then
            profit = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - (fmomega(j) + fxomega(j))
            pos_maxpi = maxloc(profit)
            posqomega2(j) = pos_maxpi(1)
        end if
         
    end do
    !$OMP END do
    !$OMP END PARALLEL
    

    
    ! calculate new import shares, skills, etc
    ! ----------------------------------------------------------------------
    Fl = 0.5d0 
    
    do j = 1, sizef
		select case(discreted2(j))
        case(0)
            outprice2(j) = 0.d0
            tsales2(j) = 0.d0
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
            skilla2(j) = 0.d0
            skillm2(j) = 0.d0
        case(1)
            outprice2(j) = markup*priceqh(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
        case(2)
            outprice2(j) = markup*newpqa(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            exp_share2(j) = 0.
            imp_share2(j) = ( newpqf(posqomega2(j))/newpqa(posqomega2(j)) )**(1.-mysigma)
        case(3)
            outprice2(j) = markup*priceqh(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = 0.
        case(4)
            outprice2(j) = markup*newpqa(posqomega2(j))**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = ( newpqf(posqomega2(j))/newpqa(posqomega2(j)) )**(1.-mysigma)
        end select
        
        if (discreted(j) > 0) then
            outprice2(j) = outprice2(j)/zbarq(posqomega2(j))
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla2(j) = skillq(posqomega2(j))
            Fu = 1. - skilla2(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm2(j) = skillq(posqomega2(j)) + pil
            if (isnan(skillm2(j) )) then
                skillm2(j) = 1.
                pil = 1./skilla2(j) - 1.d0
            end if            
        end if
    end do
    tsales2 = (womega*tsales2)/absorption
    
    ! compare to original choices 
    ! -------------------------------------------------------
    print *, 'partial equilibrium responses to tariff change of', dtariff
    print *, ' ---------------------------------------------------- '
    
    ! entry
    print *, 'entry', count((discreted==0) .and. (discreted2>0))/real(sizef - nexit)
    
    ! import dummy
    print *, 'import dummy', count( ((discreted==1) .or. (discreted==3)) .and. ((discreted2==2) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! import intensity
    print *, 'import intensity', sum(imp_share2 - imp_share)/(sizef - nexit) 
    
    ! export dummy
    print *, 'export dummy', count( ((discreted==1) .or. (discreted==2)) .and. ((discreted2==3) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! export intensity
    print *, 'export intensity', sum(exp_share2 - exp_share)/(sizef - nexit) 
    
    ! skill share
    print *, 'average increase in skill share (actual)', sum(skilla2 - skilla)/(sizef - nexit)
    print *, 'average increase in skill share (measur)', sum(skillm2 - skillm)/(sizef - nexit)
    
    ! wages
    mwage = skilla*skpremium + (1.-skilla)
    mwage2 = skilla2*skpremium + (1.-skilla2)
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(mwage2) - log(mwage)
    elsewhere
        auxv = 0.d0
    end where
    print *, 'average increase in wage', sum(auxv)/(sizef - nexit) 
        
    ! output prices
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(outprice2) - log(outprice)
    elsewhere
        auxv = 0.d0
    end where    
    print *, 'average increase in price', sum(auxv)/(sizef - nexit) 
    
    ! kmeasures
    where ((discreted>0) .and. (discreted2>0))
        kmeasure2 = log(chiq(posqomega2)) + (1.-mysigma)*log(zbarq(posqomega2)) - log(chiq(posqomega)) - (1.-mysigma)*log(zbarq(posqomega))
    elsewhere
        kmeasure2 = 0.d0
    end where    
    print *, 'k-measure', sum(kmeasure2)/(sizef - nexit) 
    print *, ' '
    
    
end subroutine partial2


! ==================================================================================
! displays fixed costs and importers and exporters market shares in domestic input markets
! ==================================================================================
    subroutine mktshares
    use crosssec
    use parameters_qf
    use parameters_estimate
    use random_draw
    implicit none
    
    integer(4) :: j, jj
    real(8), dimension(sizef) :: msales, myprice, mbuy   ! material sales, material purchase shifter
    real(8) :: firm_imp, A, B, C, checkA                   ! firm imports
    integer(4), dimension(sizef) :: auxv
    
    ! for material purchases
    ! ----------------------------
    integer(4), dimension(sizef) :: qskill
    logical, dimension(sizef) :: ish, isl
    real(8) :: htoh, htol, ltoh, ltol
    real(8), dimension(sizeq, sizeq) :: iotable
    real(8), dimension(sizeq) :: qvdiff, phivo
    
    ! ===============================================
    ! outcomes by firm: sales, import and export intensity only
    ! ===============================================
    do j = 1, sizef
		select case(discreted(j))
        case(0)
            msales(j) = 0.d0
            myprice(j) = 0.d0
            mbuy(j) = 0.d0
        case(1)
            myprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            msales(j) = myprice(j)**(1.-mysigma)*intermediatechi(posqomega(j))
            mbuy(j) = priceqh(posqomega(j))**(mysigma-1.)*tsales(j)*(1.-myalpha)/markup      ! shifter for total 
        case(2)
            myprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            msales(j) = myprice(j)**(1.-mysigma)*intermediatechi(posqomega(j))
            mbuy(j) = priceqa(posqomega(j))**(mysigma-1.)*tsales(j)*(1.-myalpha)/markup
        case(3)
            myprice(j) = markup*priceqh(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            msales(j) = myprice(j)**(1.-mysigma)*intermediatechi(posqomega(j))
            mbuy(j) = priceqh(posqomega(j))**(mysigma-1.)*tsales(j)*(1.-myalpha)/markup
        case(4)
            myprice(j) = markup*priceqa(posqomega(j))**(1.-myalpha)*wq(posqomega(j))**myalpha/zqomega(j, posqomega(j))
            msales(j) = myprice(j)**(1.-mysigma)*intermediatechi(posqomega(j))
            mbuy(j) = priceqa(posqomega(j))**(mysigma-1.)*tsales(j)*(1.-myalpha)/markup
        end select
        
    end do

    ! Normalized sales, domestic
    ! ----------------------------------------------------------------------------
    firm_imp = sum(imp_share*tsales)*(1.-myalpha)/markup
    
    A = sum(tsales)*(1.-myalpha)/markup - firm_imp
    checkA = sum(msales)
    auxv = 0; where(discreted>1) auxv = 1 
    B = sum(msales*auxv)
    C = sum(tsales*auxv)*(1.-myalpha)/markup - firm_imp
    where (discreted == 0) tsales = 0.d0
    
    print *, ' ---------------------------------------------------'
    print *, ' market shares of importers and exporters'
    print *, 'A = Home absorption of inputs, chk', A/checkA-1.
    print *, 'B = Home sales of inputs by imp&exp, B/A', B/A
    print *, 'C = Home purc. of inputs by imp&exp, C/A', C/A
    print *, 'D = sales of goods by imp&exp/total', sum(tsales*(1.-exp_share)*auxv)/sum(tsales*(1.-exp_share))
    print *, ' '
    print *, ' ---------------------------------------------------'
    print *, 'moments on fixed importing and exporting costs'
    print *, 'sales and fixed costs to convert to US$'
    print *, 'average total sales', sum(tsales)/(sizef-nexit)
    auxv = 0; where((discreted==2) .or. (discreted==4)) auxv = 1
    print *, 'average fm paid    ', sum(fmomega*auxv)/sum(auxv)
    auxv = 0; where((discreted>=3)) auxv = 1
    print *, 'average fx paid    ', sum(fxomega*auxv)/sum(auxv)
    print *, 'average fm all     ', sum(fmomega)/sizef
    print *, 'average fx all     ', sum(fxomega)/sizef
    print *, ' '
    
    ! ==================================================================
    ! patterns of material purchases, skill x unskill intensive firms
    
    do j = 1, sizeq
        qvdiff = qgrid - nu*qgrid(j)
        call phiv0(sizeq, qvdiff, phivo)
        iotable(j,:) = phivo
    end do
    
    call quartiles(skilla, qskill)
    ish = .false.
    isl = .false.
    where (qskill == 4) ish = .true.
    where ((qskill >=1) .and. (qskill<4)) isl = .true.
    
    myprice = myprice**(1.-mysigma)*womega
    where(discreted == 0) myprice = 0.d0
    
    htoh=0.d0; htol=0.d0; ltoh=0.d0; ltol=0.d0
    do j = 1, sizef ! buyer
        do jj = 1, sizef ! seller
            if ( (ish(j)) .and. (ish(jj)) ) then
                htoh = htoh + myprice(jj)*iotable(posqomega(j),posqomega(jj))*mbuy(j)    
            else if ( (ish(j)) .and. (isl(jj)) ) then
                ltoh = ltoh + myprice(jj)*iotable(posqomega(j),posqomega(jj))*mbuy(j)    
            else if ( (isl(j)) .and. (ish(jj)) ) then
                htol = htol + myprice(jj)*iotable(posqomega(j),posqomega(jj))*mbuy(j)    
            else if ( (isl(j)) .and. (isl(jj)) ) then
                ltol = ltol + myprice(jj)*iotable(posqomega(j),posqomega(jj))*mbuy(j)    
            end if
        end do
    end do
    
    
    print *, ' ---------------------------------------------------'
    print *, ' domestic material purch. check', (htoh+ltoh+htol+ltol)/A - 1.
    print *, " A = share of skilled firms' purchases from other skilled firms", htoh/(htoh+ltoh)
    print *, " B = share of unskilled firms' purchases from skilled firms    ", htol/(htol+ltol)
    print *, " difference = A/B - 1",  htoh/(htoh+ltoh)/(htol/(htol+ltol)) - 1.
    
    
    end subroutine mktshares