! ============================================================	
! ============================================================	
! polytope.F90      minimizes a function using the polytope method
! ==========================================================
! ==========================================================

subroutine polytope1p23(xinitial, v, fv)
    use parameters_polytope1
	use numerical_libraries	
	implicit none
	
	! external variables
	real(8), dimension(n1), intent(in) :: xinitial
	real(8), dimension(n1, np1), intent(out) :: v
	real(8), dimension(np1), intent(out) :: fv

	! internal variables
	real(8), dimension(n1) :: x, xin
	integer(4) :: func_evals, itercount
	real(8) :: fvguess				!accumulates results from each polytope
	real(8), dimension(n1) :: vguess
	integer(4), dimension(np1) :: permute	!permutation matrix
	character (len = 30):: how, success
	character (len = 40):: out_success
	integer(4):: j		!counts how many times polytope has been run
    logical :: cosucc
    real(8), dimension(n) :: xout

	! ============================================================	
	! SET INITIAL VALUES
	! ==========================================================
	x = xinitial
	xin = xinitial
	v = 0.d0
	fv = 0.d0

	! =================================================================
	! RUN FIRST POLYTOPE
	! =================================================================
    ! calculate function and store results with initial guess
	v(:, 1) = xin
	call calc_obj1p23(x, xout, fv(1), cosucc)
    
	! construct simplex
	call ctr_simplex1p23(xin, v, fv)
	
	! sort results
	permute = (/ (j, j=1,np1) /)
	call DSVRGP(np1, fv, fv, permute)		!sorts fv returning both sorted vector fv and permutation vector
	call DPERMA(n1, np1, v, n1, permute, two, v, n1)	!permutates the columns of v according to the sorting of fv

	! =======================================================================================
	! run polyloop
	how = "initial"
	itercount = 1
	func_evals = np1
	
	call poly_loop1p23(func_evals, itercount, v, fv, how, success)
	j = 1
	! =======================================================================================

	vguess = v(:, 1)
	fvguess = fv(1)

	! =================================================================
	! CYCLE POLYTOPE (RUN IT REPEATEDLY)
	! =================================================================

	repeat_poly1: do
		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		! NOTE: if (fvguess < fv(1)), the program also stops
		if ((j > 1) .and. (((abs(fvguess - fv(1)))<= ptol_fun) .and. (maxval(abs(v(:, 1) - vguess))<= tol_par)) .or. (fvguess < fv(1))) then
			out_success = "across polytope minimum found"
			exit repeat_poly1
		else if (j >= maxj) then
			out_success = "across polytope minimum not found"
			exit repeat_poly1
		end if

		! =============================================================
		! PRINT RESULTS AFTER EACH POLYTOPE 
		! =============================================================

			print *, "number of times polytope was run", j, fv(1)
			call print_fvv1(v(:,1), fv(1))
            xmin1 = v(:,1)
		
		! ============================================================	
		! SET INITIAL VALUES
		! ==========================================================
		vguess = v(:, 1)
		fvguess = fv(1)

		xin = v(:, 1)		!initial guess
		x = xin
		v = 0.d0
		fv = 0.d0
		
		v(:, 1) = vguess
		fv(1) = fvguess

		! DO POLYTOPE
		! construct simplex
		call ctr_simplex1p23(xin, v, fv)
		
		! sort results
		permute = (/ (j, j=1,np1) /)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(n1, np1, v, n1, permute, two, v, n1)	!permutates the columns of v according to the sorting of fv

		! run polyloop
		how = "initial"
		itercount = 1
		func_evals = np1
		call poly_loop1p23(func_evals, itercount, v, fv, how, success)
		j =	j + 1

		! =======================================================================================
		

	end do repeat_poly1
	print *, "number of times polytope was run", j, fv(1)

end subroutine polytope1p23

! ============================================================	
! ============================================================	
! END POLYTOPE EXECUTION
! (secondary subroutines below)
! ==========================================================
! ==========================================================

	subroutine ctr_simplex1p23(xin, v, fv)
		use parameters_polytope1
		use parameters_quality
        use parameters_firm
        implicit none
		
		! external variables
		real(8), dimension(n1), intent(in) :: xin
		real(8), dimension(n1, n1+1), intent(inout) :: v
		real(8), dimension(n1+1), intent(inout) :: fv

		! internal variables
		real(8) :: y(n1), xout(n), mino
		integer(4) :: j
        logical :: cosucc
        logical, parameter :: pflag2 = .false.
		
        integer(4), dimension(sizef) :: oldposq
	    integer(4), dimension(sizef) :: olddiscd	
        real(8), dimension(sizeq) :: oldpq
	    real(8), dimension(sizeq) :: oldchiq	
		
        ! ===========================================================
        ! use original choices and aggregates to estimate each deviation
        oldposq = posqomega; olddiscd = discreted; oldpq = priceqh; oldchiq = intermediatechi  
		mino = fv(1)
        
        if (pflag2) print *, 'construction of simplex'
        ! ------------------------------------------------------
        do j = 1, n1
			posqomega = oldposq; discreted = olddiscd; priceqh = oldpq; intermediatechi = oldchiq 
            
            ! new guess of parameters
            y = xin
			if (y(j) /= 0) then
                y(j) = (1 + usual_delta(j))*y(j)
            else
				y(j) = zero_term_delta 
            end if
            
            ! calculate objective and repeat with smalle change in parameter if unsuccessful
			call calc_obj1p23(y, xout, fv(j+1), cosucc)
            if (.not. cosucc) then
                print *, 'second attempt of constructing simplex'
                posqomega = oldposq; discreted = olddiscd; priceqh = oldpq; intermediatechi = oldchiq 
                y(j) = (y(j) + xin(j))/2.
                call calc_obj1p23(y, xout, fv(j+1), cosucc)
                print *, j, fv(j+1), cosucc
            end if
            if (pflag2) print *, j, fv(j+1)
            v(1:n1, j+1) = y
            
            if (fv(j+1) < mino) then
                mino = fv(j+1)
                call xallto123(xout, xmin1, xmin2, xmin3)
            end if
        end do
        
        if (pflag2) then
            print *, 'algorithm stoped after constructing simplex 1p23'
            stop
        end if
        
	end subroutine	ctr_simplex1p23

! =========================================================================================
! =========================================================================================

	subroutine poly_loop1p23(func_evals, itercount, v, fv, how, success)
		use parameters_polytope1
		implicit none
	
		integer(4), intent(inout) :: func_evals, itercount
		real(8), dimension(n1, np1), intent(inout) :: v
		real(8), dimension(np1), intent(inout) :: fv
		character (len = 30), intent(out) :: how, success

		! internal variables
		integer(4), dimension(np1) :: permute	!permutation matrix
		real(8), dimension(n1) :: xbar, xe, xr, xc, xcc
		real(8) :: fxr, fxe, fxc, fxcc
		integer(4) :: j
        logical :: cosucc
        real(8) :: oldobj, xout(n), mino

        oldobj = fv(2)
        mino = fv(1)
        cosucc = .false.
        
        while_loop1: do

		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		if ((maxval(abs(fv(2:np1) - fv(1:n1)))<= tol_fun) .and. (maxval(abs(v(:, 2:np1) - v(:, 1:n1)))<= tol_par)) then
			success = "minimum found"
			exit while_loop1
		else if ((func_evals >= maxfun) .or. (itercount >= maxiter)) then
			success = "minimum not found"
			exit while_loop1
        end if
        
        ! record results
        !if ( (cosucc) .and. (fv(1) < oldobj) ) then 
         if ( (fv(1) < oldobj) ) then 
            call print_tresult1(v(:,1), fv(1))
            oldobj = fv(1)
        end if
                
		! ================= END STOP COUNTROLS ========================

		xbar = sum(v(:, 1:n1), 2)/n1
		xr = (1 + prho)*xbar - prho*v(:, np1)
				
		call calc_obj1p23(xr, xout, fxr, cosucc)
        if (fxr < mino) then; mino = fxr; call xallto123(xout, xmin1, xmin2, xmin3); end if
		func_evals = func_evals + 1
		
		if1: if (fxr <= fv(1)) then
			! CALCULATE EXPANSION POINT
			xe = (1 + prho*pchi)*xbar - prho*pchi*v(:, np1)
					
			call calc_obj1p23(xe, xout, fxe, cosucc)
            if (fxe < mino) then; mino = fxe; call xallto123(xout, xmin1, xmin2, xmin3); end if
			func_evals = func_evals + 1
			if (fxe < fxr) then
				v(:, np1) = xe;
				fv(np1) = fxe;
				how = "expand"
			else
				v(:, np1) = xr;
				fv(np1) = fxr;
				how = "reflect"
			end if
		else	!(fv(1) <= fxr)
			if2: if (fxr < fv(n1)) then
				v(:, np1) = xr
				fv(np1) = fxr
				how = "reflect"
			else !if2: fxr >= fv(n1)
				
				! PERFORM CONTRACTION
				if31: if (fxr < fv(np1)) then
					! PERFORM OUTSIDE CONTRACTION
					xc = (1 + prho*ppsi)*xbar - prho*ppsi*v(:, np1)
							
					call calc_obj1p23(xc, xout, fxc, cosucc)
                    if (fxc < mino) then; mino = fxc; call xallto123(xout, xmin1, xmin2, xmin3); end if
					func_evals = func_evals + 1
					if41: if (fxc <= fxr) then
						v(:, np1) = xc
						fv(np1) = fxc
						how = "contract outside"
					else !if41
						! PERFORM A SHRINK
						how = "shrink"
					end if if41
				else !if31
					! PERFORM INSIDE CONTRACTION
					xcc = (1 - ppsi)*xbar + ppsi*v(:, np1)
					
					call calc_obj1p23(xcc, xout, fxcc, cosucc)
                    if (fxcc < mino) then; mino = fxcc; call xallto123(xout, xmin1, xmin2, xmin3); end if
					func_evals = func_evals + 1
					if42: if (fxcc < fv(np1)) then
						v(:, np1) = xcc
						fv(np1) = fxcc
						how = "contract inside"
					else !if42
						! PERFORM A SHRINK
						how = "shrink"
					end if if42
				end if if31
				if32: if (how == "shrink") then
					do j = 2, np1
						v(:, j) = v(:, 1) + psigma*(v(:, j) - v(:, 1))
						call calc_obj1p23(v(:, j), xout, fv(j), cosucc)
                        if (fv(j) < mino) then; mino = fv(j); call xallto123(xout, xmin1, xmin2, xmin3); end if
					end do 
					func_evals = func_evals + 1
				end if if32
			end if if2							
		end if if1
		! ================ SORT V AND FV ===========================
		permute = (/ (j, j=1,np1) /)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(n1, np1, v, n1, permute, two, v, n1)	!permutates the columns of v according to the sorting of fv
        xmin1 = v(:, 1)        
		! ================ END SORT V AND FV ========================
		itercount = itercount + 1

        end do while_loop1

	end subroutine poly_loop1p23
