!****************************************************************************
!	auxiliary subroutines for firmout and moments
!****************************************************************************

! count the number of importers, exporters and exiting firms
! ===================================================================
subroutine  count_firms
    use crosssec
    use parameters_firm
    implicit none
        
    ! -------------------------
    nexit = count(discreted == 0)
    nimport = count( (discreted == 2) .or. (discreted == 4) ) 
    nexport = count( (discreted == 3) .or. (discreted == 4) ) 
end subroutine count_firms


! vector quartile of sizef indicates the quartile of the firm in var1
! =======================================================================    
subroutine quartiles(var1, quartile)
    use parameters_firm
	use numerical_libraries
	implicit none

	real(8), dimension(sizef), intent(in) :: var1
	integer(4), dimension(sizef), intent(out) :: quartile

    ! internal variables
	integer(4) :: j, nobs
    integer(4), dimension(sizef) :: permute
    real(8), dimension(sizef) :: tvar
            
    ! construct vectors without missing observations
    tvar = var1
    nobs = sizef - nexit
    
    ! sort vector and store positions in permute
    permute = (/ (j, j=1,sizef) /)
	call DSVRGP(sizef, tvar, tvar, permute)		!sorts tvar returning both sorted vector tvar and permutation vector
    
    quartile = 4
    quartile(1:nexit) = 0
    quartile(nexit + 1 : nexit + nobs/4 + 1) = 1
    quartile(nexit + nobs/4 + 2 : nexit + 2*nobs/4 + 1) = 2
    quartile(nexit + 2*nobs/4 + 2 : nexit + 3*nobs/4 + 1) = 3
        
	tvar(permute) = tvar
    quartile(permute) = quartile
    
    if (.not. all(tvar==var1)) then
        print*, 'there is an error in quintiles'
        stop
    end if
   
end subroutine quartiles

 ! returns a matrix of the percentage of firms in each quartile combination
 ! =====================================================================================  
    subroutine match_quartile(q1, q2, qmat)
	use numerical_libraries
    use parameters_firm
    use crosssec
	implicit none

	integer(4), dimension(sizef), intent(in) :: q1, q2
    real(8), dimension(4,4) :: qmat

    ! internal variables
	integer(4) :: j1, j2, nobs
    
    ! -----------------------------------------------------------------
    ! quartiles cannot be calculated if missing observations do not coincide
    if (any( ((q1==0) .and. (q2>0)) .or. ((q1>0) .and. (q2==0)) )) then
        print *, 'quartiles cannot be matched, match_quartile in aux_firmsmoments.f90'
        !do j1 = 1, sizef
        !    if ( ((q1(j1)==0) .and. (q2(j1)>0)) .or. ((q1(j1)>0) .and. (q2(j1)==0)) ) print *, j1, q1(j1), q2(j1), discreted(j1),  hsales(j1),  avgwage(j1)
        !end do
        stop
    end if
    
    nobs = sizef - nexit  ! number of active firms
    
    do j1 = 1,4
        do j2 = 1,4
            qmat(j1,j2) = count( (q1 == j1) .and. (q2 == j2) )
        end do
    end do
    qmat = qmat/real(nobs)
        
    end subroutine match_quartile
    
!! ****************************************************************
! returns a vector indicating the quintile 1,...,5 in each observation in var1
! quintile = 0 means NaN (exit)
!! ****************************************************************
  
    subroutine pdistribution(var1, percentile)
    use parameters_firm
	use numerical_libraries
	implicit none

	real(8), dimension(sizef), intent(in) :: var1
    real(8), dimension(5), intent(out) :: percentile

    ! internal variables
	integer(4) :: j, nobs
    real(8), dimension(sizef) :: tvar
            
    ! construct vectors without missing observations
    tvar = var1     
    nobs = sizef - nexit
    
    ! sort vector and store positions in permute
    call DSVRGN(sizef, tvar, tvar)		!sorts tvar returning both sorted vector tvar and permutation vector
    percentile(1) = tvar(nexit + nobs/10.)
    percentile(2) = tvar(nexit + nobs/4.)
    percentile(3) = tvar(nexit + nobs/2.)
    percentile(4) = tvar(nexit + nobs*3./4.)
    percentile(5) = tvar(nexit + nobs*9./10.)
    
    if ( any( percentile(2:5) < percentile(1:4) ) ) then
        print *, 'error 1 in calculating percentiles, pdistribution.f90'
        stop
    elseif ( any( isnan(percentile) .or. isnan(percentile/percentile) .or. isnan(percentile*0.) ) ) then
        print *, 'error 2 in calculating percentiles, pdistribution.f90', percentile
        print *, "nexits, nobs", nexit, nobs
        !call print_par
    !    stop
    !elseif (any( percentile(2:5) == percentile(1:4) ) ) then
    !    print *, 'WARNING: PERCENTILES IN PDISTRIBUTION ARE EQUAL (in skill itensity if there is no variance of quality choice)'  ! note: this will occur for skill intensity if there is no variance on the choice of quality
    end if
    
    end subroutine pdistribution
    
    ! calculates moments in model relating imports, exports and skill distribution to each quartile of domestic sales
    ! -------------------------------------------------------------------------------------
    subroutine byqsize1(qsize, exint_model, imint_model, exshr_model, imshr_model)
        use parameters_firm
        use crosssec
        implicit none
        
        integer(4), dimension(sizef), intent(in):: qsize
        real(8), dimension(4), intent(out) :: exint_model, imint_model, exshr_model, imshr_model
        real(8), dimension(sizef) :: auxv
        integer :: j
        ! -------------------------------------
        
        do j = 1, 4
            auxv = 0.d0
            where (qsize == j) auxv = 1.d0
            exshr_model(j) = count( (qsize == j) .and. (discreted >= 3) )/real(count(qsize == j))
            imshr_model(j) = count( (qsize == j) .and. ((discreted == 2) .or. (discreted == 4)) )/real(count(qsize == j))
            exint_model(j) = sum( auxv*exp_share ) / sum(auxv)          ! unweighted sum of export share
            imint_model(j) = sum( auxv*imp_share ) / sum(auxv)          ! unweighted sum of import share
        end do
        
    end subroutine byqsize1
    
    