!! ****************************************************************
!	OLS regression 
!! ****************************************************************
  
    subroutine ols(tobs, nvar, yvector, xmat, coeff, vcoeff)
	use numerical_libraries
	implicit none

	integer(4), intent(in) :: tobs, nvar					! total number of observations (possibly with missing variables), number of variables (including constant)
	real(8), dimension(tobs), intent(in) :: yvector			! vector of dependent variable
	real(8), dimension(tobs,nvar), intent(in) :: xmat		! matrix of independent variables, excludes constant
	real(8), dimension(nvar), intent(out) :: coeff			! estimated coefficients
	real(8), dimension(nvar, nvar), intent(out) :: vcoeff	! variance of coefficients

    ! internal variables
	! to construct vector and matrix without missing observations
	integer(4), dimension(tobs) :: isnum
	real(8), dimension(:), allocatable :: tyvector			! temporary y vector, without missing observations
	real(8), dimension(:,:), allocatable :: txmat			! temporary x matrix, without missing observations
	
	
	integer(4) :: j, jj, nobs
	real(8) :: s2											! s^2 = e'e
	real(8), dimension(nvar, nvar) :: xtx					! X'X and (X'X)^-1
	real(8), dimension(nvar) ::	xty							! X'y
	real(8), dimension(:), allocatable :: error
	
	! ------------------------------------------

	! construct vector and matrix without missing observations
	! ------------------------------------------
    isnum = 1
    where ( isnan(yvector) .or. isnan(yvector*0.) .or.  isnan(sum(xmat,2)) .or. isnan(sum(xmat*0.,2)) ) isnum = 0
    nobs = sum(isnum)
    allocate( tyvector(nobs), txmat(nobs,nvar), error(nobs) )
    
    jj = 1
    do j = 1, tobs
        if (isnum(j) == 1) then
            tyvector(jj) = yvector(j)
            txmat(jj,:) = xmat(j,:)
            jj = jj + 1
        end if
    end do
    if (abs(jj - 1 - nobs)>0) then
        print *, "there is an error in calculating OLS"
        stop
    end if

	! estimate of coefficient
	! ------------------------------------------
	call DMXTXF(nobs, nvar, txmat, nobs, nvar, xtx, nvar)	! (X'X)
	call DLINRG(nvar, xtx, nvar, xtx, nvar)  				! (X'X)^-1
	j = 2
	call DMURRV(nobs, nvar, txmat, nobs, nobs, tyvector, j, nvar, xty)		! x'y
	j = 1
	call DMURRV(nvar, nvar, xtx, nvar, nvar, xty, j, nvar, coeff)			! coeff = (X'X)^{-1} X'y

    ! estimate variance of coefficient
	! ------------------------------------------
	j = 1
	call DMURRV(nobs, nvar, txmat, nobs, nvar, coeff, j, nobs, error)
	error = tyvector - error
	s2 = sum(error**2.)/(nobs-nvar)
	vcoeff = s2*xtx

    end subroutine ols

!! ****************************************************************
!	OLS regression - without calculating variance of coefficient
!! ****************************************************************
  
    subroutine ols_novar(tobs, nvar, yvector, xmat, coeff)
	use numerical_libraries
	implicit none

	integer(4), intent(in) :: tobs, nvar					! total number of observations (possibly with missing variables), number of variables (including constant)
	real(8), dimension(tobs), intent(in) :: yvector			! vector of dependent variable
	real(8), dimension(tobs,nvar), intent(in) :: xmat		! matrix of independent variables, excludes constant
	real(8), dimension(nvar), intent(out) :: coeff			! estimated coefficients
	
    ! internal variables
	! to construct vector and matrix without missing observations
	integer(4), dimension(tobs) :: isnum
	real(8), dimension(:), allocatable :: tyvector			! temporary y vector, without missing observations
	real(8), dimension(:,:), allocatable :: txmat			! temporary x matrix, without missing observations
	
	
	integer(4) :: j, jj, nobs
	real(8) :: s2											! s^2 = e'e
	real(8), dimension(nvar, nvar) :: xtx					! X'X and (X'X)^-1
	real(8), dimension(nvar) ::	xty							! X'y
	!real(8), dimension(:), allocatable :: error
	
	! ------------------------------------------

	! construct vector and matrix without missing observations
	! ------------------------------------------
    isnum = 1
    where ( isnan(yvector) .or. isnan(yvector*0.) .or.  isnan(sum(xmat,2)) .or. isnan(sum(xmat*0.,2)) ) isnum = 0
    nobs = sum(isnum)
    allocate( tyvector(nobs), txmat(nobs,nvar) )
    
    jj = 1
    do j = 1, tobs
        if (isnum(j) == 1) then
            tyvector(jj) = yvector(j)
            txmat(jj,:) = xmat(j,:)
            jj = jj + 1
        end if
    end do
    if (abs(jj - 1 - nobs)>0) then
        print *, "there is an error in calculating OLS"
        stop
    end if

	! estimate of coefficient
	! ------------------------------------------
	call DMXTXF(nobs, nvar, txmat, nobs, nvar, xtx, nvar)	! (X'X)
	call DLINRG(nvar, xtx, nvar, xtx, nvar)  				! (X'X)^-1
	j = 2
	call DMURRV(nobs, nvar, txmat, nobs, nobs, tyvector, j, nvar, xty)		! x'y
	j = 1
	call DMURRV(nvar, nvar, xtx, nvar, nvar, xty, j, nvar, coeff)			! coeff = (X'X)^{-1} X'y

    
    end subroutine ols_novar
