! PURPOSE: to calculate out-of-sample moments on prices and skills
! ===========================================================================
subroutine outofsample
    use parameters_estimate
    use parameters_quality
    use parameters_firm
    use crosssec
    use data_moments
    implicit none
    
    ! normalized total sales, employment, average wage and khandelwal's quality measure
    real(8), dimension(sizef) :: employment, avgwage, kmeasure        
    integer(4), dimension(sizef) :: qsize
    real(8), dimension(sizef) :: auxv
    real(8), dimension(4) :: ksize_model        ! average khandelwal measure by quartile of firm size
    real(8), dimension(4), parameter :: ksize_data = (/-0.585, -0.068, 0.466, 1.042/)       ! k_imp_ai
    integer(4) :: j, jj
    real(8) :: imps_model, exps_model
    real(8) :: firm_imp, cimp, pkmeasure_model(5), avgk
    
    ! price regressions
    integer(4) :: tobs
    integer(4), parameter :: nvar = 2
    real(8), dimension(:), allocatable  :: yvector
    real(8), dimension(:,:), allocatable :: xmat
    real(8), dimension(nvar) :: outcoeff       ! regression results: coefficients
    real(8), dimension(nvar, nvar) :: outvar
    
    ! import patterns
    integer(4), parameter :: nivar = 3
    real(8), dimension(nivar) :: dicoeff, sicoeff       ! regression results: coefficients
    real(8), dimension(nivar, nivar) :: divar, sivar
    
    
    print *, ' '
    print *, '---------------------------------------------------------'
    print *, 'OUT OF SAMPLE MOMENTS'
    print *, '---------------------------------------------------------'
    
    ! aggregate trade shares
    ! --------------------------------------------------------
    firm_imp = sum(imp_share*tsales*womega)*(1.-myalpha)/markup
    cimp = (pcf/pca)**(1.-mysigma)*income
    imps_model = (cimp + firm_imp)/absorption/(1.+tariff)
    exps_model = sum(exp_share*tsales*womega)/absorption
    print*, 'international trade/absorption, data, model'
    print*, 'imports', imps_data, imps_model
    print*, 'exports', exps_data, exps_model
    print *, ' '
    print *, '---------------------------------------------------------'
    
    ! aggregate skill intensity and skill premium
    ! --------------------------------------------------------
    where (discreted == 0)
        employment = 0.d0
    elsewhere
        avgwage = skilla*skpremium + (1. - skilla)
        employment = (myalpha/markup)*tsales/avgwage
    end where
    aggska = sum(employment*skilla)/sum(employment)
    
    print *, 'aggregate skills' ! new baseline, jan 27 in february files
    print *, ' '
    print *, 'measured, data (avg 1982-88), model' 
    print *, 'intensity', skshare_data, aggskm 
    print *, 'premium  ', skpremium_data, aggskpm
    print *, ' '
    print *, 'actual, data (avg 1982-88, Attanasio et al.), model' 
    print *, 'intensity    0.085', aggska
    print *, 'premium    1.8-2.6', skpremium
    print *, ' '
    print *, '---------------------------------------------------------'
    
    !price regressions & khandelwal measures
    ! -----------------------------------------------------------------
    where (discreted == 0)
        kmeasure = 0.d0
    elsewhere
        kmeasure = log(tsales*(1.-exp_share)) - (1.-mysigma)*log(outprice)
    end where
    kmeasure = kmeasure - sum(kmeasure)/(sizef - nexit)     ! demean
    where (discreted == 0) kmeasure = 0.d0
    
    ! unconditional distribution of k-measures
    call pdistribution(kmeasure, pkmeasure_model)
    
    
    print *, ' ----------------------------------------------------- '
    print *, 'distribution of kmeasures, model'
    print *, '10% ', pkmeasure_model(1)
    print *, '25% ', pkmeasure_model(2)
    print *, '50% ', pkmeasure_model(3)
    print *, '75% ', pkmeasure_model(4)
    print *, '90% ', pkmeasure_model(5)
    print *, 'avg, standard deviation = ', sum(kmeasure)/(sizef - nexit), sum((kmeasure)**2.)/(sizef - nexit)
    print *, ' '
     
    ! khandelwal measure by quartile of firm domestic sales
    call quartiles(log(nhsales), qsize)
    do j = 1, 4
        auxv = 0.d0
        where (qsize == j) auxv = 1.d0
        ksize_model(j) = sum( auxv*kmeasure )/sum(auxv)          ! sum( auxv*firmlwh ) / sum( auxv*(firml) )    
    end do
    print *, 'khandelwal measure of quality by quartiles of size'
    print *, '1 ', ksize_data(1), ksize_model(1)
    print *, '2 ', ksize_data(2), ksize_model(2)
    print *, '3 ', ksize_data(3), ksize_model(3)
    print *, '4 ', ksize_data(4), ksize_model(4)
    print *, ' '
    print *, " ---------------------------------------- "
    print *, 'price regressions' ! new baseline, jan 27 in february files
    print *, ' '
    tobs = sizef - nexit
    allocate(yvector(tobs), xmat(tobs,2))
    xmat(:,1) = 1.d0
    
    ! measured skill intensity (with standard errors)
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            yvector(jj) = log(outprice(j))
            xmat(jj,2) = skillm(j)
        end if
    end do
    yvector = yvector - sum(yvector)/real(tobs)     ! demean price vectors
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on (skill intensity)'
        print*, 'output coef', skill_pdata(1), outcoeff(2)
        print*, 'output se  ', skill_sepdata(1), outvar(2,2)**0.5
        print *, ' '
    end if
    
    ! ln sales
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            xmat(jj,2) = log(tsales(j))
        end if
    end do
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on ln(sales)'
        print*, 'output coef', sales_pdata(1), outcoeff(2)
        print*, 'output se  ', sales_sepdata(1), outvar(2,2)**0.5
        print *, ' '
    end if
    
    ! ---------------------------------------------------------------------
    ! khandelwal measure
    ! ---------------------------------------------------------------------
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            xmat(jj,2) = kmeasure(j)
        end if
    end do
    call ols(tobs, nvar, yvector, xmat, outcoeff, outvar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'coefficient, ln(unit price) on k-measures'
        print*, 'output coef', kimpai_pdata(1), outcoeff(2)
        print*, 'output se  ', kimpai_sepdata(1), outvar(2,2)**0.5
        print *, ' '
    end if  
    
    ! -----------------------------------------------------
    ! importing patterns
    ! -----------------------------------------------------
    print *, " ---------------------------------------- "
    print *, 'importing patterns' ! new baseline, jan 27 in february files
    print *, ' '
    
    ! regression of import dummy
    ! ---------------------------------------------------------------------
    deallocate(yvector, xmat)
    tobs = sizef - nexit
    allocate(yvector(tobs), xmat(tobs,nivar))
    xmat(:,1) = 1.d0
    jj = 0
    do j = 1, sizef
        if(discreted(j) > 0) then
            jj = jj + 1
            if ( (discreted(j)==2) .or. (discreted(j)==4)) then
                yvector(jj) = 1.d0
            else
                yvector(jj) = 0.d0
            end if
            xmat(jj,2) = skillm(j)
            xmat(jj,3) = log(tsales(j))
        end if
    end do
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nivar, yvector, xmat, dicoeff, divar)      ! output prices on skill intensity
    
    
    ! regression of import share
    ! ---------------------------------------------------------------------
    deallocate(yvector, xmat)
    tobs = nimport
    allocate(yvector(tobs), xmat(tobs,nivar))
    xmat(:,1) = 1.d0
    jj = 0
    do j = 1, sizef
        if( (discreted(j)==2) .or. (discreted(j)==4)) then
            jj = jj + 1
            yvector(jj) = log(imp_share(j))
            xmat(jj,2) = skillm(j)
            xmat(jj,3) = log(tsales(j))
        end if
    end do
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nivar, yvector, xmat, sicoeff, sivar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'regression of import dummy on skill intensity and sales'
        print*, 'skill coef', 0.099d0, dicoeff(2)
        print*, 'skill se  ', 0.010d0, divar(2,2)**0.5
        print*, 'n obs.     46,770', sizef - nexit
        print*, 'regression of import shares on skill intensity and sales'
        print*, 'skill coef', 0.144, sicoeff(2)
        print*, 'skill se  ', 0.012, sivar(2,2)**0.5
        print*, 'n obs.     12,041', nimport
        print *, ' '
    end if

    ! regression of import dummy, excluding exporters
    ! ---------------------------------------------------------------------
    deallocate(yvector, xmat)
    tobs = sizef - nexit - nexport
    allocate(yvector(tobs), xmat(tobs,nivar))
    xmat(:,1) = 1.d0
    jj = 0
    do j = 1, sizef
        if( (discreted(j) == 1 ) .or. (discreted(j) == 2) ) then
            jj = jj + 1
            if ( (discreted(j)==2) .or. (discreted(j)==4)) then
                yvector(jj) = 1.d0
            else
                yvector(jj) = 0.d0
            end if
            xmat(jj,2) = skillm(j)
            xmat(jj,3) = log(tsales(j))
        end if
    end do
    if (jj/= tobs) print *, 'there is a mistake in number of observations, see moments.f90'
    call ols(tobs, nivar, yvector, xmat, dicoeff, divar)      ! output prices on skill intensity
    
    if (toprint) then
        print*, 'regression of import dummy excluding exporters (model)'
        print*, 'skill coef', dicoeff(2)
        print*, 'skill se  ', divar(2,2)**0.5
        print*, 'n obs.    ', sizef - nexit - nexport
    end if 
    
    ! partial equilibrium response to a change in import tariffs
    ! ========================================================================
    call partial2
     
end subroutine outofsample

! ========================================================================
! estimate partial equilibrium response to a change in import tariffs
! ========================================================================
subroutine  partial
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use crosssec
    use random_draw
	implicit none

	integer(4), dimension(sizef) :: posqomega2, discreted2
    
    real(8), dimension(sizeq, 4) :: profit
    integer, dimension(2) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8) :: fprofit
    real(8) :: newpqf, newpqa
    real(8), dimension(sizef) :: outprice2, skilla2, skillm2, imp_share2, exp_share2, mwage, mwage2, tsales2, auxv
    real(8) :: Fu, Fl, pil
    real(8), parameter:: dtariff = -0.1d0
    
    ! -------------------------------------------------------------------
    ! change in import tariffs
    newpqf = priceqf*(1.d0+dtariff)
    newpqa = (priceqh**(1.-mysigma) + newpqf**(1.-mysigma))**(1./(1.-mysigma))
    
   ! --------------------------------------------------------------------    
    !$OMP PARALLEL shared(posqomega2, discreted2), private(profit, pos_maxpi, fprofit)
    !$OMP do
    do j = 1, sizef
        ! calculate profit for firm j
         profit(:, 1) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq
         profit(:, 2) = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq - fmomega(j)
         profit(:, 3) = (1./mysigma)*( ( markup*priceqh**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - fxomega(j)
         profit(:, 4) = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - (fmomega(j) + fxomega(j))
         
         ! update its choice
         fprofit = maxval(profit)
         pos_maxpi = maxloc(profit)
         if (fprofit >=0.d0) then
            posqomega2(j) = pos_maxpi(1)
            discreted2(j) = pos_maxpi(2)
         else
            posqomega2(j) = -1
            discreted2(j) = 0
         end if
         
    end do
    !$OMP END do
    !$OMP END PARALLEL
    
    ! calculate new import shares, skills, etc
    ! ----------------------------------------------------------------------
    Fu = exp(1./varpi)/(1. + exp(1./varpi))        ! truncation of logistic
    Fl = 0.5d0 
    
    do j = 1, sizef
		select case(discreted2(j))
        case(0)
            outprice2(j) = 0.d0
            tsales2(j) = 0.d0
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
            skilla2(j) = 0.d0
            skillm2(j) = 0.d0
        case(1)
            outprice2(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
        case(2)
            outprice2(j) = markup*newpqa**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            exp_share2(j) = 0.
            imp_share2(j) = ( newpqf/newpqa )**(1.-mysigma)
        case(3)
            outprice2(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = 0.
        case(4)
            outprice2(j) = markup*newpqa**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = ( newpqf/newpqa )**(1.-mysigma)
        end select
        
        if (discreted(j) > 0) then
            outprice2(j) = outprice2(j)/zbarq(posqomega2(j))
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla2(j) = skillq(posqomega2(j))
            Fu = 1. - skilla2(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm2(j) = skillq(posqomega2(j)) + pil
            if (isnan(skillm2(j) )) then
                skillm2(j) = 1.
                pil = 1./skilla2(j) - 1.d0
            end if            
        end if
    end do
    tsales2 = (womega*tsales2)/absorption
    
    ! compare to original choices 
    ! -------------------------------------------------------
    print *, 'partial equilibrium responses to tariff change of', dtariff
    print *, ' ---------------------------------------------------- '
    
    ! entry
    print *, 'entry', count((discreted==0) .and. (discreted2>0))/real(sizef - nexit)
    
    ! import dummy
    print *, 'import dummy', count( ((discreted==1) .or. (discreted==3)) .and. ((discreted2==2) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! import intensity
    print *, 'import intensity', sum(imp_share2 - imp_share)/(sizef - nexit) 
    
    ! export dummy
    print *, 'export dummy', count( ((discreted==1) .or. (discreted==2)) .and. ((discreted2==3) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! export intensity
    print *, 'export intensity', sum(exp_share2 - exp_share)/(sizef - nexit) 
    
    ! skill share
    print *, 'average increase in skill share (actual)', sum(skilla2 - skilla)/(sizef - nexit)
    print *, 'average increase in skill share (measur)', sum(skillm2 - skillm)/(sizef - nexit)
    
    ! wages
    mwage = skilla*skpremium + (1.-skilla)
    mwage2 = skilla2*skpremium + (1.-skilla2)
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(mwage2) - log(mwage)
    elsewhere
        auxv = 0.d0
    end where
    print *, 'average increase in wage', sum(auxv)/(sizef - nexit) 
        
    ! output prices
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(outprice2) - log(outprice)
    elsewhere
        auxv = 0.d0
    end where    
    print *, 'average increase in price', sum(auxv)/(sizef - nexit) 
    print *, ' '
    
end subroutine partial


! ========================================================================
! estimate partial equilibrium response to a change in import tariffs, when firms cannot change discrete choices
! ========================================================================
subroutine  partial2
    use numerical_libraries
    use parameters_quality
    use parameters_firm
	use parameters_qf
	use parameters_estimate
    use crosssec
    use random_draw
	implicit none

	integer(4), dimension(sizef) :: posqomega2, discreted2
    
    real(8), dimension(sizeq) :: profit
    integer, dimension(1) :: pos_maxpi       ! position of maximum profits
    integer(4) :: j, jj
    
    real(8) :: fprofit
    real(8) :: newpqf, newpqa
    real(8), dimension(sizef) :: outprice2, skilla2, skillm2, imp_share2, exp_share2, mwage, mwage2, tsales2, auxv
    real(8) :: Fu, Fl, pil
    real(8), parameter:: dtariff = -0.1d0
    
    ! -------------------------------------------------------------------
    ! change in import tariffs
    newpqf = priceqf*(1.d0+dtariff)
    newpqa = (priceqh**(1.-mysigma) + newpqf**(1.-mysigma))**(1./(1.-mysigma))
    discreted2 = discreted
   ! --------------------------------------------------------------------    
    !$OMP PARALLEL shared(posqomega2, discreted2), private(profit, pos_maxpi, fprofit)
    !$OMP do
    do j = 1, sizef
        if ((discreted(j) <=1) .or. (discreted(j) == 3)) then
            posqomega2(j) = posqomega(j)
        else if (discreted(j) == 2) then
            ! calculate profit for firm j
            profit = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*chiq - fxq - fmomega(j)
            pos_maxpi = maxloc(profit)
            posqomega2(j) = pos_maxpi(1)
        else if (discreted(j) == 4) then
            profit = (1./mysigma)*( ( markup*newpqa**(1.-myalpha)/zqomega(j,:) )**(1.-mysigma) )*(chiq + chiqf) - fxq - (fmomega(j) + fxomega(j))
            pos_maxpi = maxloc(profit)
            posqomega2(j) = pos_maxpi(1)
        end if
         
    end do
    !$OMP END do
    !$OMP END PARALLEL
    

    
    ! calculate new import shares, skills, etc
    ! ----------------------------------------------------------------------
    Fl = 0.5d0 
    
    do j = 1, sizef
		select case(discreted2(j))
        case(0)
            outprice2(j) = 0.d0
            tsales2(j) = 0.d0
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
            skilla2(j) = 0.d0
            skillm2(j) = 0.d0
        case(1)
            outprice2(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            imp_share2(j) = 0.d0
            exp_share2(j) = 0.d0
        case(2)
            outprice2(j) = markup*newpqa**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*chiq(posqomega2(j))
            exp_share2(j) = 0.
            imp_share2(j) = ( newpqf/newpqa )**(1.-mysigma)
        case(3)
            outprice2(j) = markup*priceqh**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = 0.
        case(4)
            outprice2(j) = markup*newpqa**(1.-myalpha)/zqomega(j, posqomega2(j))
            tsales2(j) = outprice2(j)**(1.-mysigma)*(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            exp_share2(j) = chiqf(posqomega2(j))/(chiq(posqomega2(j)) + chiqf(posqomega2(j)))
            imp_share2(j) = ( newpqf/newpqa )**(1.-mysigma)
        end select
        
        if (discreted(j) > 0) then
            outprice2(j) = outprice2(j)/zbarq(posqomega2(j))
            ! calculations for measured skill intensity
            ! truncated logistic  x = pibar + varpi*log(p/(1-p)) where p = (Fu-Fl)*rd + Fl due to truncation.
            skilla2(j) = skillq(posqomega2(j))
            Fu = 1. - skilla2(j)
            Fu = exp(Fu/varpi)/(1. + exp(Fu/varpi))        ! truncation of logistic
            pil = varpi*log( ( (Fu - Fl)*random_skill(j) + Fl ) / ( 1. - (Fu - Fl)*random_skill(j) - Fl) )
            skillm2(j) = skillq(posqomega2(j)) + pil
            if (isnan(skillm2(j) )) then
                skillm2(j) = 1.
                pil = 1./skilla2(j) - 1.d0
            end if            
        end if
    end do
    tsales2 = (womega*tsales2)/absorption
    
    ! compare to original choices 
    ! -------------------------------------------------------
    print *, 'partial equilibrium responses to tariff change of', dtariff
    print *, ' ---------------------------------------------------- '
    
    ! entry
    print *, 'entry', count((discreted==0) .and. (discreted2>0))/real(sizef - nexit)
    
    ! import dummy
    print *, 'import dummy', count( ((discreted==1) .or. (discreted==3)) .and. ((discreted2==2) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! import intensity
    print *, 'import intensity', sum(imp_share2 - imp_share)/(sizef - nexit) 
    
    ! export dummy
    print *, 'export dummy', count( ((discreted==1) .or. (discreted==2)) .and. ((discreted2==3) .or. (discreted2==4)) )/real(sizef - nexit)
    
    ! export intensity
    print *, 'export intensity', sum(exp_share2 - exp_share)/(sizef - nexit) 
    
    ! skill share
    print *, 'average increase in skill share (actual)', sum(skilla2 - skilla)/(sizef - nexit)
    print *, 'average increase in skill share (measur)', sum(skillm2 - skillm)/(sizef - nexit)
    
    ! wages
    mwage = skilla*skpremium + (1.-skilla)
    mwage2 = skilla2*skpremium + (1.-skilla2)
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(mwage2) - log(mwage)
    elsewhere
        auxv = 0.d0
    end where
    print *, 'average increase in wage', sum(auxv)/(sizef - nexit) 
        
    ! output prices
    where ((discreted>0) .and. (discreted2>0))
        auxv = log(outprice2) - log(outprice)
    elsewhere
        auxv = 0.d0
    end where    
    print *, 'average increase in price', sum(auxv)/(sizef - nexit) 
    print *, ' '
    
end subroutine partial2