! ============================================================	
! ============================================================	
! polytope.F90      minimizes a function using the polytope method
! ==========================================================
! ==========================================================

subroutine polytopeci(xinitial, v, fv)
    use parameters_polytopeci
	use numerical_libraries	
	implicit none
	
	! external variables
	real(8), dimension(nci), intent(in) :: xinitial
	real(8), dimension(nci, np1), intent(out) :: v
	real(8), dimension(np1), intent(out) :: fv

	! internal variables
	real(8), dimension(nci) :: x, xin
	integer(4) :: func_evals, itercount
	real(8) :: fvguess				!accumulates results from each polytope
	real(8), dimension(nci) :: vguess
	integer(4), dimension(np1) :: permute	!permutation matrix
	character (len = 30):: how, success
	character (len = 40):: out_success
	integer(4):: j		!counts how many times polytope has been run
    logical :: cosucc

	! ============================================================	
	! SET INITIAL VALUES
	! ==========================================================
	x = xinitial
	xin = xinitial
	v = 0.d0
	fv = 0.d0

	! =================================================================
	! RUN FIRST POLYTOPE
	! =================================================================
    ! calculate function and store results with initial guess
	v(:, 1) = xin
	call calc_objci(x, fv(1), cosucc)
    
	! construct simplex
	call ctr_simplexci(xin, v, fv)
	
	! sort results
	permute = (/ (j, j=1,np1) /)
	call DSVRGP(np1, fv, fv, permute)		!sorts fv returning both sorted vector fv and permutation vector
	call DPERMA(nci, np1, v, nci, permute, two, v, nci)	!permutates the columns of v according to the sorting of fv

	! =======================================================================================
	! run polyloop
	how = "initial"
	itercount = 1
	func_evals = np1
	
	call poly_loopci(func_evals, itercount, v, fv, how, success)
	j = 1
	! =======================================================================================

	vguess = v(:, 1)
	fvguess = fv(1)

	! =================================================================
	! CYCLE POLYTOPE (RUN IT REPEATEDLY)
	! =================================================================

	repeat_polyci: do
		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		! NOTE: if (fvguess < fv(1)), the program also stops
		if ((j > 1) .and. (((abs(fvguess - fv(1)))<= ptol_fun) .and. (maxval(abs(v(:, 1) - vguess))<= tol_par)) .or. (fvguess < fv(1))) then
			out_success = "across polytope minimum found"
			exit repeat_polyci
		else if (j >= maxj) then
			out_success = "across polytope minimum not found"
			exit repeat_polyci
		end if

		! =============================================================
		! PRINT RESULTS AFTER EACH POLYTOPE 
		! =============================================================

			print *, "number of times polytope was run", j
			print *, "v(:, 1)"
			print "(f15.5, f15.5, f15.5, f15.5)", v(:, 1)
			print *, "fv(1)"
			print "(f15.5)", fv(1)
			call print_fvvci(v(:,1), fv(1))
		
		! ============================================================	
		! SET INITIAL VALUES
		! ==========================================================
		vguess = v(:, 1)
		fvguess = fv(1)

		xin = v(:, 1)		!initial guess
		x = xin
		v = 0
		fv = 0
		
		v(:, 1) = vguess
		fv(1) = fvguess

		! DO POLYTOPE
		! construct simplex
		call ctr_simplexci(xin, v, fv)
		
		! sort results
		permute = (/ (j, j=1,np1) /)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(nci, np1, v, nci, permute, two, v, nci)	!permutates the columns of v according to the sorting of fv

		! run polyloop
		how = "initial"
		itercount = 1
		func_evals = np1
		call poly_loopci(func_evals, itercount, v, fv, how, success)
		j =	j + 1

		! =======================================================================================
		

	end do repeat_polyci
	print *, "number of times polytope was run", j
	print *, "v(:, 1) after several polytope loops"
	print "(f15.5, f15.5, f15.5, f15.5)", v(:, 1)
	print *, "fv(1) after several polytope loops"
	print "(f15.5)", fv(1)

end subroutine polytopeci

! ============================================================	
! ============================================================	
! END POLYTOPE EXECUTION
! (secondary subroutines below)
! ==========================================================
! ==========================================================

	subroutine ctr_simplexci(xin, v, fv)
		use parameters_polytopeci
		use parameters_quality
        use parameters_firm
        implicit none
		
		! external variables
		real(8), dimension(nci), intent(in) :: xin
		real(8), dimension(nci, nci+1), intent(inout) :: v
		real(8), dimension(nci+1), intent(inout) :: fv

		! internal variables
		real(8), dimension(nci) :: y
		integer(4) :: j
        logical :: cosucc
		
        integer(4), dimension(sizef) :: oldposq
	    integer(4), dimension(sizef) :: olddiscd	
        real(8), dimension(sizeq) :: oldchiq	
		
        ! ===========================================================
        ! use original choices and aggregates to estimate each deviation
        oldposq = posqomega; olddiscd = discreted; oldchiq = intermediatechi  
		
        if (pflag) print *, 'construction of simplex'
        ! ------------------------------------------------------
        do j = 1, nci
			posqomega = oldposq; discreted = olddiscd; intermediatechi = oldchiq 
            
            ! new guess of parameters
            y = xin
			if (y(j) /= 0) then
                y(j) = (1 + usual_delta)*y(j)
            else
				y(j) = zero_term_delta 
            end if
            
            ! calculate objective and repeat with smalle change in parameter if unsuccessful
			call calc_objci(y, fv(j+1), cosucc)
            if (.not. cosucc) then
                print *, 'second attempt of constructing simplex'
                posqomega = oldposq; discreted = olddiscd; intermediatechi = oldchiq 
                y(j) = (y(j) + xin(j))/2.
                call calc_objci(y, fv(j+1), cosucc)
                print *, j, fv(j+1), cosucc
            end if
            if (pflag) print *, j, fv(j+1)
            v(1:nci, j+1) = y
        end do
        
        if (pflag) then
            print *, 'algorithm stoped after constructing simplex'
            stop
        end if
        
	end subroutine	ctr_simplexci

! =========================================================================================
! =========================================================================================

	subroutine poly_loopci(func_evals, itercount, v, fv, how, success)
		use parameters_polytopeci
		implicit none
	
		integer(4), intent(inout) :: func_evals, itercount
		real(8), dimension(nci, np1), intent(inout) :: v
		real(8), dimension(np1), intent(inout) :: fv
		character (len = 30), intent(out) :: how, success

		! internal variables
		integer(4), dimension(np1) :: permute	!permutation matrix
		real(8), dimension(nci) :: xbar, xe, xr, xc, xcc
		real(8) :: fxr, fxe, fxc, fxcc
		integer(4) :: j
        logical :: cosucc
        real(8) :: oldobj

        oldobj = fv(2)
        cosucc = .false.
        
        while_loop1: do

		! =============================================================
		! STOP COUNTROLS
		! =============================================================
		if ((maxval(abs(fv(2:np1) - fv(1:nci)))<= tol_fun) .and. (maxval(abs(v(:, 2:np1) - v(:, 1:nci)))<= tol_par)) then
			success = "minimum found"
			exit while_loop1
        else if (fv(1)<= 1.e-10) then
			success = "minimum found"
			exit while_loop1
		else if ((func_evals >= maxfun) .or. (itercount >= maxiter)) then
			success = "minimum not found"
			exit while_loop1
        end if
        
        ! record results
        if ( (cosucc) .and. (fv(1) < oldobj) ) then 
            call print_tresultci(v(:,1), fv(1))
            oldobj = fv(1)
        end if
                
		! ================= END STOP COUNTROLS ========================

		xbar = sum(v(:, 1:nci), 2)/nci
		xr = (1 + prho)*xbar - prho*v(:, np1)
				
		call calc_objci(xr, fxr, cosucc)
		func_evals = func_evals + 1
		
		if1: if (fxr <= fv(1)) then
			! CALCULATE EXPANSION POINT
			xe = (1 + prho*pchi)*xbar - prho*pchi*v(:, np1)
					
			call calc_objci(xe, fxe, cosucc)
			func_evals = func_evals + 1
			if (fxe < fxr) then
				v(:, np1) = xe;
				fv(np1) = fxe;
				how = "expand"
			else
				v(:, np1) = xr;
				fv(np1) = fxr;
				how = "reflect"
			end if
		else	!(fv(1) <= fxr)
			if2: if (fxr < fv(nci)) then
				v(:, np1) = xr
				fv(np1) = fxr
				how = "reflect"
			else !if2: fxr >= fv(nci)
				
				! PERFORM CONTRACTION
				if31: if (fxr < fv(np1)) then
					! PERFORM OUTSIDE CONTRACTION
					xc = (1 + prho*ppsi)*xbar - prho*ppsi*v(:, np1)
							
					call calc_objci(xc, fxc, cosucc)
					func_evals = func_evals + 1
					if41: if (fxc <= fxr) then
						v(:, np1) = xc
						fv(np1) = fxc
						how = "contract outside"
					else !if41
						! PERFORM A SHRINK
						how = "shrink"
					end if if41
				else !if31
					! PERFORM INSIDE CONTRACTION
					xcc = (1 - ppsi)*xbar + ppsi*v(:, np1)
					
					call calc_objci(xcc, fxcc, cosucc)
					func_evals = func_evals + 1
					if42: if (fxcc < fv(np1)) then
						v(:, np1) = xcc
						fv(np1) = fxcc
						how = "contract inside"
					else !if42
						! PERFORM A SHRINK
						how = "shrink"
					end if if42
				end if if31
				if32: if (how == "shrink") then
					do j = 2, np1
						v(:, j) = v(:, 1) + psigma*(v(:, j) - v(:, 1))
						call calc_objci(v(:, j), fv(j), cosucc)
					end do 
					func_evals = func_evals + 1
				end if if32
			end if if2							
		end if if1
		! ================ SORT V AND FV ===========================
		permute = (/ (j, j=1,np1) /)
		call DSVRGP(np1, fv, fv, permute)				!sorts fv returning both sorted vector fv and permutation vector
		call DPERMA(nci, np1, v, nci, permute, two, v, nci)	!permutates the columns of v according to the sorting of fv
		! ================ END SORT V AND FV ========================
		itercount = itercount + 1

        end do while_loop1

	end subroutine poly_loopci

    ! -------------------------------------------------------------
    subroutine print_tresultci(x, obj)
		use parameters_polytopeci
		use parameters_quality
		use parameters_firm
        implicit none

        real(8), dimension(nci), intent(in) :: x
        real(8), intent(in) :: obj
        integer(4) :: j
        logical, parameter :: auxl = .true.
        
    open(1, FILE='tcounteri.txt', STATUS='REPLACE')
		write(1,*), obj
        write(1,*), auxl
        do j = 1, nci
			write(1,*), x(j)
		end do
		do j = 1, sizef
		    write(1,*), posqomega(j)
        end do
        do j = 1, sizef
		    write(1,*), discreted(j)
	    end do
        do j = 1, sizeq
		    write(1,*), intermediatechi(j)
        end do
	close(1)


    end subroutine print_tresultci

    ! -------------------------------------------------------------
    subroutine print_fvvci(x, obj)
		use parameters_polytopeci
		use parameters_quality
		use parameters_firm
        implicit none

        real(8), dimension(nci), intent(in) :: x
        real(8), intent(in) :: obj
        integer(4) :: j
        logical, parameter :: auxl = .true.
              
     open(1, FILE='counteri_fvv.txt', STATUS='REPLACE')
		write(1,*), obj
        write(1,*), auxl
        do j = 1, nci
			write(1,*), x(j)
		end do
		do j = 1, sizef
		    write(1,*), posqomega(j)
        end do
        do j = 1, sizef
		    write(1,*), discreted(j)
	    end do
        do j = 1, sizeq
		    write(1,*), intermediatechi(j)
        end do
	close(1)

	end subroutine print_fvvci